%% Defines the buses for the interface!

MagnetometerInput = Simulink.Bus;

mag_in_x = Simulink.BusElement;
mag_in_x.Name = 'x';
mag_in_x.DataType = 'int16';

mag_in_y = Simulink.BusElement;
mag_in_y.Name = 'y';
mag_in_y.DataType = 'int16';

mag_in_z = Simulink.BusElement;
mag_in_z.Name = 'z';
mag_in_z.DataType = 'int16';

MagnetometerInput.Elements = [mag_in_x mag_in_y mag_in_z];


% Camera interface!

CameraInputPoint = Simulink.Bus;

cam_input_point = Simulink.BusElement;
cam_input_point.Name = 'point';
cam_input_point.DataType = 'single';
cam_input_point.Dimensions = 2;

cam_input_valid = Simulink.BusElement;
cam_input_valid.Name = 'valid';
cam_input_valid.DataType = 'boolean';

CameraInputPoint.Elements = [cam_input_point cam_input_valid ];

 
CamInput = Simulink.Bus;

cam_input_pos = Simulink.BusElement;
cam_input_pos.Name = 'pos';
cam_input_pos.DataType = 'CameraInputPoint';
cam_input_pos.Dimensions = 1;

cam_input_aref = Simulink.BusElement;
cam_input_aref.Name = 'aref';
cam_input_aref.DataType = 'CameraInputPoint';
cam_input_aref.Dimensions = 1;

CamInput.Elements = [cam_input_pos cam_input_aref];


% Internal position and angle validity interface
PosMeasurement = Simulink.Bus;

PosMeasurement_pos = Simulink.BusElement;
PosMeasurement_pos.Name = 'pos';
PosMeasurement_pos.Dimensions = 2;

PosMeasurement_valid = Simulink.BusElement;
PosMeasurement_valid.Name = 'valid';
PosMeasurement_valid.DataType = 'boolean';

PosMeasurement.Elements = [PosMeasurement_pos PosMeasurement_valid];


AngleMeasurement = Simulink.Bus;

AngleMeasurement_angle = Simulink.BusElement;
AngleMeasurement_angle.Name = 'angle';

AngleMeasurement_valid = Simulink.BusElement;
AngleMeasurement_valid.Name = 'valid';
AngleMeasurement_valid.DataType = 'boolean';

AngleMeasurement.Elements = [AngleMeasurement_angle AngleMeasurement_valid];


% Controller Input Bus

cIn =Simulink.Bus;

cIn_allRobots = Simulink.BusElement;
cIn_allRobots.Name = 'all_robots';
cIn_allRobots.DataType = 'Bus: CameraInputPoint';
cIn_allRobots.Dimensions = 4% maxRobots.Value;

cIn_position = Simulink.BusElement;
cIn_position.Name = 'position';
cIn_position.Dimensions = 2;

cIn_obstacle = Simulink.BusElement;
cIn_obstacle.Name = 'obstacle';
cIn_obstacle.Dimensions = 3;
cIn_obstacle.DataType = 'Bus: CameraInputPoint';

cIn_agentIndex = Simulink.BusElement;
cIn_agentIndex.Name = 'agent_index';
cIn_agentIndex.DataType = 'int32';

cIn_target = Simulink.BusElement;
cIn_target.Name = 'target';
cIn_target.DataType = 'Bus: CameraInputPoint';

cIn.Elements = [cIn_allRobots
                cIn_position
                cIn_obstacle
                cIn_agentIndex
                cIn_target];

% Run config
RunConfig = Simulink.Bus;

RunConfig_nRobots = Simulink.BusElement;
RunConfig_nRobots.Name = 'nRobots';

RunConfig_activateLeaderV = Simulink.BusElement;
RunConfig_activateLeaderV.Name = 'activateLeaderV';
RunConfig_activateLeaderV.DataType = 'boolean';

RunConfig_forceObstacleDeactivate = Simulink.BusElement;
RunConfig_forceObstacleDeactivate.Name = 'forceObstacleDeactivate';
RunConfig_forceObstacleDeactivate.DataType = 'boolean';

RunConfig_maxRobots = Simulink.BusElement;
RunConfig_maxRobots.Name = 'maxRobots';

RunConfig_formation = Simulink.BusElement;
RunConfig_formation.Name = 'formation';
RunConfig_formation.DataType = 'Enum: Formation';

RunConfig_formationRotation = Simulink.BusElement;
RunConfig_formationRotation.Name = 'formationRotation';

RunConfig_controllerType = Simulink.BusElement;
RunConfig_controllerType.Name = 'controllerType';
RunConfig_controllerType.DataType = 'Enum: ControllerType';


RunConfig.Elements = [RunConfig_nRobots 
                      RunConfig_activateLeaderV 
                      RunConfig_forceObstacleDeactivate
                      RunConfig_formation
                      RunConfig_formationRotation
                      RunConfig_controllerType];
