%% Log plotting
% This file plots all relevant information from the latest log, 
% without going via Simulink
%


% Run init to gather logs to timeseries.
Simulator_log_init;

% Add to datastruct array
log = Simulink.SimulationData.Dataset;

log = log.addElement(simdata);
log = log.addElement(arefdata);
log = log.addElement(aref_valid);
log = log.addElement(pos_valid);
log = log.addElement(observed_pos);
log = log.addElement(observed_heading);
log = log.addElement(obstacle);
log = log.addElement(target);
log = log.addElement(nsb_v_target);
log = log.addElement(nsb_v_obstacle);
log = log.addElement(nsb_active_obstacle);
log = log.addElement(nsb_v);

realLog = Simulink.SimulationData.Dataset;
for i = 1:log.numElements
    entry = log.getElement(i);
    
    signal = Simulink.SimulationData.Signal;
    signal.Values = entry;
    signal.Name = entry.Name;
    realLog = realLog.addElement(signal);
end

log = realLog;

%% Do most of the plotting.

% Position
% Plot position in x-y grid
figure(2); clf; hold on;

plot(log.get('obs_pos').Values.Data(:,1), log.get('obs_pos').Values.Data(:,2), 'b');
plot(log.get('meas_pos').Values.Data(:,1), log.get('meas_pos').Values.Data(:,2), '*');
plot(log.get('meas_aref').Values.Data(:,1), log.get('meas_aref').Values.Data(:,2), 'y*');
legend('Log observed', 'Measured', 'Measured aref');

text(log.get('meas_pos').Values.Data(1,1), log.get('meas_pos').Values.Data(1,2), 'start')

% Plot obstacle
%plot(log.get('obstacle').Values.Data(:,1), log.get('obstacle').Values.Data(:,2), 'g*');

% Plot target
plot2d(log.get('target').Values, 'c*');

axis equal;

set(gca, 'XDir', 'reverse');

% Plot position in subsystems
% + means

figure(4); clf; hold on;



for i = 1:2
    subplot(2,1,i); hold on; grid on;
   
    plot(log.get('obs_pos').Values.Time, log.get('obs_pos').Values.Data(:,i), 'b');
    plot(log.get('meas_pos').Values.Time, log.get('meas_pos').Values.Data(:,i), '*');
   
    
    thePos = log.get('meas_pos').Values.Data(:,i);
    theTime = log.get('meas_pos').Values.Time;
    invalid = thePos(log.get('pos_valid').Values.Data == 0);
    invalidTime = theTime(log.get('pos_valid').Values.Data == 0);
    
    plot(invalidTime, invalid, '*r');
    
    legend( 'Observed', 'Measured');
end

%%
%%
CTRLconfig.N = 1;
CTRLconfig.obstacle_d = 0.3;
CTRLconfig.collision_d = 0.3;


exportStruct.doExport = 0;
exportStruct.name     = 'Comoro_nsb';
exportStruct.timeSlots= [12.6];
exportStruct.doExtra  = [1];




%plotSimulation(CTRLconfig, log, figure(1), exportStruct);