function [ M ] = plotSimulation(CTRLconfig, log, fig, exportStruct )
%PLOTSIMULATION Summary of this function goes here
%   Detailed explanation goes here

% BIG NB::::
% This file is now assumed to only be used on the log-parts on the actual
% beaglebones.
% That is, I will print Test, not simulated at...


%% PLOTTING

%
% Setup
%
% OKEEI, are we dealing with multiple logs?

multipleLogs = 0;
if iscell(log) 
   % we got a cell array. Rename and rebrand
   cLogArray = log;
   log = cLogArray{1};
   
   multipleLogs = 1;
   fprintf('Multiple log stuff: %d! \n', length(cLogArray));
end

% figureName

if nargin < 4
    exportFigure     = 0;       % True/false
else
    exportFigure     = exportStruct.doExport;
    exportFigureName = exportStruct.name;
    exportTimeSlots  = exportStruct.timeSlots;
    exportDoExtra    = exportStruct.doExtra;
    
    if isfield(exportStruct, 'minBox')
        exportMinBox = exportStruct.minBox;
    else
        exportMinBox = [3 3];
    end
end
% Stop at what time?
stopTime = log.get('obs_pos').Values.Time(end);



% Just a variable namechange
data = log.get('obs_pos').Values.Data;


static_obstacles = 0;
obstacleNames = {'obstacle_1', 'obstacle_2', 'obstacle_3'};


N = CTRLconfig.N;
time = log.get('obs_pos').Values.Time;


% Start/stop stuff
statusRun = 1;

% Movie container
M = repmat(getframe, 0);

% 
% Figure Information
%

if nargin < 4
    fig = figure(1); 
end
clf; 
hold on;
set(fig, 'Tag', 'agentScreen');
set(fig, 'KeyPressFcn', @keyPress);

fig_axes = get(fig, 'CurrentAxes');
set(fig_axes, 'DrawMode', 'fast');


foundFig = findobj('Tag','agentScreen');

%
% Plot axis
%


    allX = reshape(data(1,:,:),1, size(data,2)*size(data,3));
    allY = reshape(data(2,:,:),1, size(data,2)*size(data,3));

    allX = [data(:,1)' log.get('target').Values.Data(:,1)']% log.get('obstacle').Values.Data(:,1)'];
    allY = [data(:,2)' log.get('target').Values.Data(:,2)']% log.get('obstacle').Values.Data(:,2)'];

    ratio = 1.1;

    xlim([ratio*min([allX -0.1]) ratio*max(allX)]);
    ylim([ratio*min([allY -0.1]) ratio*max(allY)]);
    
if(~isfield(exportStruct, 'plotAxis') || exportStruct.plotAxis == 1)
    line(xlim, [0 0], 'Color', 'k');
    line([0 0], ylim, 'Color', 'k');
end

%
% Setup plot information
%
hold on; axis equal;
xlabel('x');
ylabel('y');

if isfield(exportStruct, 'colors')
    colors = exportStruct.colors;
else
    colors = {'blue', 'red', 'green', 'magenta', 'cyan', 'yellow'};
end

oldPlots = zeros(1,2*N);
if multipleLogs
    oldPlots = zeros(1, 2*length(cLogArray));
end
            


oldBaryPlot = zeros(1,2);
oldObstaclePlots = zeros(1,3*length(obstacleNames));
oldTargetPlots = zeros(1,2);

oldExtraPlot = [];


% Set index to stop the drawing
stopIndex = find(time == stopTime)
startIndex = 1;
stepIndex = 1;
currentIndex = startIndex; % set by the continuePlot function
stopIndex_full = stopIndex; % used to reset stop index

% Export-counter
exportCounter = 1;

% Plot obstacles
% If ther are any obstacles

if(size(static_obstacles, 1) >= 2)

    for i = 1:size(static_obstacles, 2)
       obstacle = static_obstacles(:,i);

       t = 0:0.01:2*pi;
       radi = CTRLconfig.obstacle_d;

       plot(obstacle(1) + radi*cos(t), obstacle(2) + radi*sin(t), 'k');

    end
end

% Start the plotting
continuePlot();

    function continuePlot
        
        % remove extra plots
        if any(ishandle(oldExtraPlot))
            delete(oldExtraPlot);
            oldExtraPlot = [];
        end
        
        for t = startIndex:stepIndex:stopIndex
            currentIndex = t;
            if statusRun == 0
                
                % store info
                startIndex = t;
                
                break;
            end
            
            
            if exportFigure == 0
                pause(0.01) 
                fprintf('At time: %f \n', time(t));
            end
            

            timeFrame = 1:t;
            
            % Plot all agents
            agentLoop = 1;
            if multipleLogs
                agentLoop = 1:length(cLogArray);
            end
            
            for i = agentLoop
                if multipleLogs
                  
                    data = cLogArray{i}.get('obs_pos').Values.Data;
                end
              
                
                xpos = data(timeFrame,1);
                ypos = data(timeFrame,2);
                
                
                %
                % Remove the old plot.
                %
                if oldPlots(2*i) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    delete(oldPlots(2*i));
                    delete(oldPlots(2*i-1));
                end

                oldPlots(2*i-1) = plot(xpos, ypos, [colors{i} '--']);
                oldPlots(2*i) = plot(xpos(end), ypos(end), [colors{i} 'o']);
                %oldPlot = plot(xpos, ypos, '.');

                title(sprintf('Robot logs at time: %.2fs', time(t)));

                % Plot last position as a circle
                %plot(xpos(end), ypos(end), [colors{i} 'o']);


            end
            
            % Plot all obstacles that are positive...
            
            nObs = 3;

            
            for obs = 1:length(obstacleNames)
               xpos = log.get(obstacleNames{obs}).Values.Data(timeFrame,1);
                ypos = log.get(obstacleNames{obs}).Values.Data(timeFrame,2);
                
                xpos = mean(xpos(1:min(t, 5)));
                ypos = mean(ypos(1:min(t, 5)));
                
                if(any([xpos ypos] < 0))
                    continue;
                end
                %
                % Remove the old plot.
                %
                if oldObstaclePlots(3*obs-2) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    %delete(oldObstaclePlots(3*obs));
                    %delete(oldObstaclePlots(3*obs-1));
                    delete(oldObstaclePlots(3*obs-2));
                end
                
                
                %oldObstaclePlots(3*obs-1) = plot(xpos, ypos, [colors{obs} '*']);
                %oldObstaclePlots(3*obs) = plot(xpos(end), ypos(end), [colors{obs+1} 'o']);

                k = 0:0.01:2*pi;
                radi = CTRLconfig.obstacle_d;

                oldObstaclePlots(3*obs-2) = plot(xpos(end) + radi*cos(k), ypos(end) + radi*sin(k), 'k');

                
                
            end
            
            
            
            
            
            
            % Plot current obstacle
            
            
            
            
            
            for obs = 1 % Currently only support for 1
                continue;
                xpos = log.get('obstacle').Values.Data(timeFrame,1);
                ypos = log.get('obstacle').Values.Data(timeFrame,2);
                
                xpos = mean(xpos(1:min(t, 5)));
                ypos = mean(ypos(1:min(t, 5)));

                %
                % Remove the old plot.
                %
                if oldObstaclePlots(2*obs) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    delete(oldObstaclePlots(2*obs));
                    delete(oldObstaclePlots(2*obs-1));
                    delete(oldObstaclePlots(3));
                end

                oldObstaclePlots(2*obs-1) = plot(xpos, ypos, [colors{obs} '*']);
                oldObstaclePlots(2*obs) = plot(xpos(end), ypos(end), [colors{obs+1} 'o']);

                k = 0:0.01:2*pi;
                radi = CTRLconfig.obstacle_d;

                oldObstaclePlots(3) = plot(xpos(end) + radi*cos(k), ypos(end) + radi*sin(k), 'k');

            end
            
            % Plot current Target
            
            for obs = 1 % Currently only support for 1
                if(isfield(exportStruct, 'plotTarget') && exportStruct.plotTarget == 0)
                    continue;
                end
               
                xpos = log.get('target').Values.Data(timeFrame,1);
                ypos = log.get('target').Values.Data(timeFrame,2);

                if(any([xpos ypos] < 0))
                    continue;
                end
                %
                % Remove the old plot.
                %
                if oldTargetPlots(2*obs) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    delete(oldTargetPlots(2*obs));
                    %delete(oldTargetPlots(2*obs-1));
                end

                %oldTargetPlots(2*obs-1) = plot(xpos(1), ypos(1), [ 'c*']);
                oldTargetPlots(2*obs) = plot(xpos(end), ypos(end), [ 'c*']);



            end


            % Plot barycenter
            % Plot barycenter
           % baryX = 1/N*reshape(sum(data(1,:,timeFrame), 2), [length(timeFrame) 1]);
            %baryY = 1/N*reshape(sum(data(2,:,timeFrame), 2), [length(timeFrame) 1]);


            % Remove old plot
            if oldBaryPlot(1) ~= 0
                delete(oldBaryPlot(1))
                delete(oldBaryPlot(2))
            end

            %oldBaryPlot(1) = plot(baryX, baryY, 'k-.');
            %oldBaryPlot(2) = plot(baryX(end), baryY(end), 'kx');

            drawnow;


            %
            % Export?
            %
            if exportFigure
               % Check timeslot
               if  exportCounter <= length(exportTimeSlots) && time(t) >= exportTimeSlots(exportCounter)
                   
                   % Check if we should plot extras
                   if length(exportDoExtra) >= exportCounter
                      this_doExtra = exportDoExtra(exportCounter);
                   else
                       this_doExtra = 0;
                   end
                   
                   % Plot extra
                   if this_doExtra
                       plotExtra();
                   end
                   
                   
                   %path = 'export/';
                   
                   
                   % Export full picture
                   exportPlot(fig, sprintf('%s_%d', exportFigureName,exportCounter), 'full');
                   
                   % Export small picture
                   exportPlot(fig, sprintf('%s_%d_small', exportFigureName,exportCounter), 0.5);
                   
                   % Export small picture of where the agents are
                   oldAxis = axis;
                   
                   % "zoom" to the current positions
                   %xpos = data(1,:,currentIndex);
                   %ypos = data(2,:,currentIndex);
                   xpos = data(currentIndex,1);
                   ypos = data(currentIndex,2);
                   
                   %xlim([ratio*min([xpos]-0.1) ratio*max([xpos min(xpos)+2])]);
                   %ylim([ratio*min([ypos]-0.1) ratio*max([ypos min(ypos)+3])]);
                   
                   x = [min(xpos) max(xpos)];
                   y = [min(ypos) max(ypos)];
                   
                   xlim([min([x(1) mean(x)-exportMinBox(1)/2]), ...
                       max([x(2) mean(x)+exportMinBox(1)/2])]);
                   
                   ylim([min([y(1) mean(y)-exportMinBox(2)/2]), ...
                       max([y(2) mean(y)+exportMinBox(2)/2])]);
                   
                   %xlim([min(xpos)-exportMinBox(1)/6 max([xpos min(xpos)+exportMinBox(1)])]);
                   %ylim([min(ypos)-exportMinBox(2)/6 max([ypos min(ypos)+exportMinBox(2)])]);
                   
                   exportPlot(fig, sprintf('%s_%d_small_zoom', exportFigureName,exportCounter), 0.5);
                   exportPlot(fig, sprintf('%s_%d_zoom', exportFigureName,exportCounter), 'full');

                   %hgexport(fig, sprintf('%s%s%g.eps', path, exportFigureName, exportCounter));
                   %input('Press enter', 's')
                   
                   
                   % Restore old axis
                   axis(oldAxis);
                   
                   
                   % delete extra
                   if this_doExtra
                        if any(ishandle(oldExtraPlot))
                            delete(oldExtraPlot);
                            oldExtraPlot = [];
                        end
                   end
                   

                   % Increment the counter
                   exportCounter = exportCounter + 1;
               end

            end % end export
            
            % Testing some movie export
            %oldPos = get(fig, 'Position');            
            %set(fig, 'Position', [oldPos(1:2) 720 609]);     
            M(end+1) = getframe(fig); 
            %set(fig, 'Position', oldPos);
            
        end % End main loop
        
        % done plotting 
        statusRun = 0;
        
    end % end function
% Plot barycenter
baryX = 1/N*reshape(sum(data(1,:,:), 2), [size(data, 3) 1]);
baryY = 1/N*reshape(sum(data(2,:,:), 2), [size(data, 3) 1]);

dataLength = size(data, 3);
lookupVector = [1:ceil(dataLength/10):dataLength];

%plot(baryX(lookupVector), baryY(lookupVector), 'rx');

function plotExtra
    if any(ishandle(oldExtraPlot))
        delete(oldExtraPlot);
        oldExtraPlot = [];
    end
    
    % Plot a circle around each agent, with the correct d
    
    k = 0:0.01:2*pi;
    
     % Plot all agents
    agentLoop = 1;
    if multipleLogs
        agentLoop = 1:length(cLogArray);
    end

    for agent = agentLoop
        fprintf('Agent: %d ', agent);
        if multipleLogs
            log = cLogArray{agent};
            data = cLogArray{agent}.get('obs_pos').Values.Data;
        end
    
    
       pos = data(currentIndex, :)';
       r = CTRLconfig.collision_d;
       
       oldExtraPlot(end+1) = plot(pos(1) + r*cos(k), pos(2) + r*sin(k));
       
       % Plot velocity vector
       v                        = log.get('nsb_v').Values.Data(currentIndex,:)';
       %v_collision              = log.get('v_collision').Values.Data(:,agent,currentIndex);
       v_obstacle               = log.get('nsb_v_obstacle').Values.Data(currentIndex,:)';
       %v_formation              = log.get('v_formation').Values.Data(:,agent,currentIndex);
       %v_formation_collision    = log.get('v_formation_collision').Values.Data(:,agent,currentIndex);
       v_target                 = log.get('nsb_v_target').Values.Data(currentIndex,:)';
       
       
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v, 'b--', 'v_d'));
       %oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_collision, 'g--', ''));
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_obstacle, 'k--', 'v_o'));
       %oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_formation, 'r--', 'u'));
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_target, 'r--', 'v_t'));

       %oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_formation_collision, 'r-'));
       
       % Display information in the commandline
       %fprintf('Agent: %d \n', agent);
       %v
       
       dist = norm(pos-log.get('target').Values.Data(currentIndex,:)');
       %fprintf('Distance to target: %f \n', dist);
       
       %if log.get('collision_active').Values.Data(currentIndex, agent) == 1
       %    fprintf('\tCollision active \n');
       %end
       if log.get('nsb_active_obstacle').Values.Data(currentIndex) == 1
           fprintf('\tObstacle active \n');
       end
       
    end
    
    
    
end



function keyPress(src, eventData)
    fprintf('Pressed key: %s \n', eventData.Key);
    
    if eventData.Key == 's'
        statusRun = ~statusRun;
        
        % set correct stop time again (always full)
        stopIndex = stopIndex_full;
        
        % if starting
        if statusRun == 1
            continuePlot();
        else
           plotExtra(); 
        end
        
    end
    
    if eventData.Key == 'r'
        if statusRun == 0
            startIndex = 1;
            statusRun = 1;
            continuePlot();
        end
    end
    
    switch eventData.Key
        case 'd' % step
            if statusRun == 0
                startIndex = currentIndex;
                stopIndex = currentIndex + stepIndex;
                statusRun = 1;
                continuePlot();
                plotExtra();
            end
        case 'f' % step
            if statusRun == 0
                startIndex = currentIndex - stepIndex;
                stopIndex = currentIndex- stepIndex;
                statusRun = 1;
                continuePlot();
                plotExtra();
            end
    end
end






function out = vector_append(v, new)
    out = zeros(1, length(new));
    for i = 1:length(new)
        out(i) = new(i);
    end
    
    out = [v out];
end

function handle = plotVector(pos, v, style)
%Plots the vector from the pos
% Does magic
    
    % find vector angle
    angle = atan2(v(2), v(1));
    
    % Get rotation matrix
    R = @(a)( [cos(a) -sin(a); sin(a) cos(a)]);
    
    % Length of vector
    length = norm(v);
    
    % Calculate the vector
    vector = R(angle) * [length; 0];
    
    % Line positions
    lineX = [pos(1) pos(1) + vector(1)];
    lineY = [pos(2) pos(2) + vector(2)];
    
    % Plot the line    
    handle = line(lineX, lineY);
    
    % Set style
    t = regexp(style, '(?<color>\w*)', 'names');
    color = t.color;
    t = regexp(style, '(?<linestyle>\W*)', 'names');
    linestyle = t.linestyle;
    
    set(handle, 'Color', color);
    set(handle, 'LineStyle', linestyle);
    
    
end


function handle = plotVectorWithAnnotation(pos, v, style, theText)
    % ARROW SETUP
    tip = 0.3;
    scale = 0.06;
    arrow = [0 0;
                -tip -1;
                0 -0.7;
                tip -1;
                0 0;]';   
    
    % find vector angle
    angle = atan2(v(2), v(1));
    
    % Get rotation matrix
    R = @(a)( [cos(a) -sin(a); sin(a) cos(a)]);
    
    % Length of vector
    length = 0.2;
    length = norm(v);
    
    if(isfield(exportStruct, 'setVLength')) 
        length = exportStruct.setVLength;
    end
    

    
    
    
    % If ca 0, do not plot arrow
    if norm(v) < 0.01
        handle = [];
        return;
    end
    
    % Plot the actual line
    handle = plotVector(pos, length*v/norm(v), style);
    
    % Calculate the vector
    vector = R(angle) * [length; 0];           
    stopPos = pos + vector;

   
    % Plot the arrowhead to Stop          
    target = R(angle+pi/2+pi)*scale*arrow;
    handle(end+1) = plot(stopPos(1)+target(1,:),stopPos(2)+target(2,:),'k');
    handle(end+1) = patch(stopPos(1)+target(1,:), stopPos(2)+target(2,:), 'k');
    
    % if plot text
    if nargin > 3
       tx = mean([pos(1) stopPos(1)]);
       ty = mean([pos(2) stopPos(2)]);
       
       % Augment a bit
       %tx = tx + 0.05*sign(tx);
       ty = ty + 0.03*sign(ty);
       
       handle(end+1) = text(tx, ty, theText);
    end

end




end