%% This library file should contain all model information about the omniwheel.
% All parameters will be added to a config struct called
% OmniwheelModel

% Parameters

r_wheel = 0.025;
l = 0.12;

% System model
% nu = T*tau, tau = [omega_1, ..]


T = (r_wheel/3) * [0    sqrt(3)/2   -sqrt(3)/2;
                   1    -1/2        -1/2;
                   1/l  1/l         1/l;];
               
               
T = (r_wheel/3) * [0    (3/2)*sqrt(3)/2   -(3/2)*sqrt(3)/2;
                   2    -1          -1;
                   1/l  1/l         1/l;];
               
T = (r_wheel/3) * [0    sqrt(3)   -sqrt(3);
                   2    -2          -2;
                   1/l  1/l         1/l;];
               
T = (r_wheel/3) * [0    (3/2)*sqrt(3)/2   -(3/2)*sqrt(3)/2;
                   2    -1                -1;
                   1/l  1/l               1/l;];
               
OmniwheelModel.T = T;
OmniwheelModel.r_wheel = r_wheel;
OmniwheelModel.l = l;
               
%% Servo calibration
% The servos behave like a slacked operation
%                           |      *  maxOmega
%                           |     *
%                           |    *    maxServo
%                 ------*********------
%                      *    |   zeroOffset
%                     *     |
%                    *      |

% Research have yielded the following values:
zeroOffset = 4;     % Values of 4 and bellow results in zero movement;
maxOmega   = 20;    % Application specific

% Generate a lookuptable based on the following:
% (The table is based on linear regresion between the points)
xdata_servo = [-36 -32 -20 -4 0 4 20   32  36];
ydata_omega = [-4.8 -4.8 -3.92 0 0 0 3.92 4.8 4.8];

xdata_servo = [4 20   32  36];
ydata_omega = [0 3.92 4.8 4.8];

% In matlab, this can be accomplished by the function interp1
% e.g:
% omega_i = interp1(xdata_servo, ydata_omega, 20)


OmniwheelModel.lookup_servo = xdata_servo;
OmniwheelModel.lookup_omega = ydata_omega;

OmniwheelModel.full_lookup_servo = [-OmniwheelModel.lookup_omega(end:-1:1) 0 OmniwheelModel.lookup_omega];
OmniwheelModel.full_lookup_omega = [-OmniwheelModel.lookup_servo(end:-1:1) 0 OmniwheelModel.lookup_servo];


%% Just test omniwheel function

Servo = -40:40;
omega = Omniwheel_S2omega(OmniwheelModel, Servo);

% figure(3);
% plot(Servo, omega)
