/* 
 * Wrappers to make I/O functions available with C linkage. This allows C++
 * methods to be called from C code.
 *
 * Copyright 2009-2010 The MathWorks, Inc. */

#ifndef MATLAB_MEX_FILE

#include <inttypes.h>
#include "WProgram.h"
#include "Wire.h"
#include "Servo.h"

#ifndef __AVR_ATmega1280__
#error seriously
#endif

extern "C" {
    #include "twi.h"
}

extern "C" { 
    void __cxa_pure_virtual(void);
}

/* TwoWire Wire2 = TwoWire();*/


/* CUSTOM WIRE STUFF */
extern "C" void wire_send(int val) {Wire.send(val); }
extern "C" void wire_begin(int address) {Wire.beginTransmission(address); }
extern "C" void wire_end(void) {Wire.endTransmission();}
extern "C" uint8_t wire_receive(void) {return (uint8_t)Wire.receive();}
extern "C" void wire_requestFrom(int address, int nBytes) {Wire.requestFrom(address, nBytes);}
extern "C" uint8_t wire_available(void) {return (uint8_t)Wire.available();}


/* Setup Servo Library */
#define N_SERVOS 12 // For arduino MEGA
#define SERVO_PIN_SHIFT 2 // what pin number is servo[0] ?
#define VALID_PIN(pin) ( pin<N_SERVOS+2 ? ( pin >= SERVO_PIN_SHIFT ? 1 : 0) : 0)


// Set up servo array
Servo myServos[N_SERVOS];

extern "C" void servo_attach_c(int pin) {
    
    if(!VALID_PIN(pin))
        return;
    
    myServos[pin-SERVO_PIN_SHIFT].attach(pin);
}

extern "C" void servo_write_c(int pin, int value) {
    if(!VALID_PIN(pin))
        return;
    
    myServos[pin-SERVO_PIN_SHIFT].write(value);
}

/* Serial library */

extern "C" void Serial_custom_begin(int port, long r) 
{
#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)   
    switch(port) {
    case 0:
        Serial.begin(r); 
        break;
    case 1:
        Serial1.begin(r); 
        break;
    case 2:
        Serial2.begin(r); 
        break;  
    case 3:
        Serial3.begin(r); 
        break;    
    }
#else
        Serial.begin(r); 
#endif        
}

extern "C" void Serial_custom_read(int port, int showOutStatus, uint8_t *outData, int *outStatus) 
{ 
	int libFcnOutput;
#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)   
    switch(port) {
    case 0:
		libFcnOutput = Serial.read();
        break;
    case 1:
		libFcnOutput = Serial1.read();
        break;
    case 2:
		libFcnOutput = Serial2.read();
        break;  
    case 3:
		libFcnOutput = Serial3.read();
        break;    
    }    
#else
    libFcnOutput = Serial.read();
#endif          
    *outData = (uint8_t) libFcnOutput; 
    *outStatus = (libFcnOutput != -1);
}

extern "C" void Serial_custom_write(int port, uint8_t * c, size_t s)
{ 
#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)     
    switch(port) {
    case 0:
        Serial.write(c, s); 
        break;
    case 1:
        Serial1.write(c, s); 
        break;
    case 2:
        Serial2.write(c, s); 
        break;  
    case 3:
        Serial3.write(c, s); 
        break;    
    }    
#else
        Serial.write(c, s); 
#endif          
}



#endif /* #ifndef MATLAB_MEX_FILE */


