%% setup
addpath('../../libraries');
p = 2; % Planar

N = 4; % 3 robots


D = [-1 0 0 1 -1 0;
     1 -1 0 0 0 1;
     0  1 -1 0 1 0;
     0  0 1  -1 0 -1];
 



% Obstacles

static_obstacles = [7 2.3; 6 -0.1]';


% Initial Positions
initial_positions = [2 2; 0 0.5; 1 1; -1 -1]';


% Desired Formation
desired_formation = [0 0; 2 2; 0 2; 2 0]';


% Desired barycenter
desired_barycenter_position = [10 0]';


CTRLconfig.p = p;
CTRLconfig.N = N;
CTRLconfig.obstacle_d = 0.5;
CTRLconfig.collision_d = 0.4; 
CTRLconfig.debug = 0;
CTRLconfig.enable_guidance = 0;


 


%% Run

figure(3); clf
plotGraph(D);

sim comoro

%% Plotting




dataStruct.position = position.Data;
dataStruct.static_obstacles = static_obstacles;
dataStruct.time = position.Time;

plotSimulation(dataStruct, CTRLconfig, log, figure(1));


%% PLOTTING

data = position.Data;
time = position.Time;

figure(2); clf; hold on;

colors = {'blue', 'red', 'green', 'magenta', 'cyan', 'yellow'};
legends = {};
legendIDs = [];
for i = 1:N
    
    xpos = reshape(data(1,i,:), [size(data, 3) 1]);
    ypos = reshape(data(2,i,:), [size(data, 3) 1]);
    
    plot(xpos, ypos, [colors{i} '--']);
    
    % Plot last position as a circle
    legendIDs(end+1) = plot(xpos(end), ypos(end), [colors{i} 'o']);
    legends{end+1} = sprintf('%g', i);
    
    % Plot where it is active on collision
    collision_active_indexes = find(collision_active.Data(:,i)==1);
    a = plot(xpos(collision_active_indexes), ypos(collision_active_indexes), ['+' colors{i}]);
    %set(a, 'LineWidth', 2);
    
    % Plot a line to the current active collision agent
    
    collision_current_data = collision_current.Data;
    
    xpos_coll = reshape(collision_current_data(1,i,:), [size(collision_current_data, 3) 1]);
    ypos_coll = reshape(collision_current_data(2,i,:), [size(collision_current_data, 3) 1]);
    
    a = line([xpos(collision_active_indexes) xpos(collision_active_indexes)+xpos_coll(collision_active_indexes)]', ...
             [ypos(collision_active_indexes) ypos(collision_active_indexes)+ ypos_coll(collision_active_indexes)]');
    
    set(a, 'Color', [.7 .7 .7]);
    %set(a, 'LineStyle', '--');
    
    % CRAZY: Plot the distances..
    % only some of the indexes..
    indexes = collision_active_indexes(2000:100:length(collision_active_indexes));
    text(xpos(indexes), ypos(indexes), ...
        sprintf('%f', sqrt(xpos_coll.^2 + ypos_coll.^2)));
    %sqrt(xpos_coll.^2 + ypos_coll.^2)
end
% Plot barycenter
baryX = 1/N*reshape(sum(data(1,:,:), 2), [size(data, 3) 1]);
baryY = 1/N*reshape(sum(data(2,:,:), 2), [size(data, 3) 1]);

dataLength = size(data, 3);
lookupVector = [1:ceil(dataLength/10):dataLength];

plot(baryX(lookupVector), baryY(lookupVector), 'rx');



axis equal

% Plot axis

line(xlim, [0 0], 'Color', 'k');
line([0 0], ylim, 'Color', 'k');
legend(legendIDs, legends)

%legend('1', '2', '3', 'barycenter', 'location', 'best');

% Plot obstacles

% If ther are any obstacles

if(size(static_obstacles, 1) >= 2)

    for i = 1:size(static_obstacles, 2)
       obstacle = static_obstacles(:,i);

       t = 0:0.01:2*pi;
       radi = CTRLconfig.obstacle_d;

       plot(obstacle(1) + radi*cos(t), obstacle(2) + radi*sin(t), 'k');

    end
end
% Plot velocities
figure(3); clf

for i = 1:N
    dimensionToPlot = 1;
    
    subplot(N,p, p*(i-1)+dimensionToPlot); 
    
    plot(time, reshape(data(dimensionToPlot,i,:), [size(data, 3) 1]))
    
    dimensionToPlot = 2;
    
    subplot(N,p, p*(i-1)+dimensionToPlot); 
    
    plot(time, reshape(data(dimensionToPlot,i,:), [size(data, 3) 1]))
    
end





