function [ M ] = plotSimulation( dataStruct, CTRLconfig, log, fig, exportStruct )
%PLOTSIMULATION Summary of this function goes here
%   Detailed explanation goes here
%% PLOTTING

%
% Setup
%

% figureName

if nargin < 5
    exportFigure     = 0;       % True/false
else
    exportFigure     = exportStruct.doExport;
    exportFigureName = exportStruct.name;
    exportTimeSlots  = exportStruct.timeSlots;
    exportDoExtra    = exportStruct.doExtra;
    
    if isfield(exportStruct, 'minBox')
        exportMinBox = exportStruct.minBox;
    else
        exportMinBox = [3 3];
    end
end
% Stop at what time?
if isfield(dataStruct, 'endTime')
    stopTime = dataStruct.endTime;
else
    stopTime = 25;
end


% Just a variable namechange
data = dataStruct.position;
static_obstacles = dataStruct.static_obstacles;
N = CTRLconfig.N;
time = dataStruct.time;


% Start/stop stuff
statusRun = 1;

% Movie container
M = repmat(getframe, 0)

% 
% Figure Information
%

if nargin < 4
    fig = figure(1); 
end
clf; 
hold on;
set(fig, 'Tag', 'agentScreen');
set(fig, 'KeyPressFcn', @keyPress);

fig_axes = get(fig, 'CurrentAxes');
set(fig_axes, 'DrawMode', 'fast');


foundFig = findobj('Tag','agentScreen');

%
% Plot axis
%
allX = reshape(data(1,:,:),1, size(data,2)*size(data,3));
allY = reshape(data(2,:,:),1, size(data,2)*size(data,3));

ratio = 1.1;

xlim([ratio*min([allX -0.1]) ratio*max(allX)]);
ylim([ratio*min([allY -0.1]) ratio*max(allY)]);

line(xlim, [0 0], 'Color', 'k');
line([0 0], ylim, 'Color', 'k');

%
% Setup plot information
%
hold on; axis equal;
xlabel('x');
ylabel('y');

colors = {'blue', 'red', 'green', 'magenta', 'cyan', 'yellow'};

oldPlots = zeros(1,2*N);
oldBaryPlot = zeros(1,2);

oldExtraPlot = [];


% Set index to stop the drawing
stopIndex = find(time == stopTime)
startIndex = 1;
stepIndex = 3;
currentIndex = startIndex; % set by the continuePlot function
stopIndex_full = stopIndex; % used to reset stop index

% Export-counter
exportCounter = 1;

% Plot obstacles
% If ther are any obstacles

if(size(static_obstacles, 1) >= 2)

    for i = 1:size(static_obstacles, 2)
       obstacle = static_obstacles(:,i);

       t = 0:0.01:2*pi;
       radi = CTRLconfig.obstacle_d;

       plot(obstacle(1) + radi*cos(t), obstacle(2) + radi*sin(t), 'k');

    end
end

% Start the plotting
continuePlot();

    function continuePlot
        
        % remove extra plots
        if any(ishandle(oldExtraPlot))
            delete(oldExtraPlot);
            oldExtraPlot = [];
        end
        
        for t = startIndex:stepIndex:stopIndex
            currentIndex = t;
            if statusRun == 0
                
                % store info
                startIndex = t;
                
                break;
            end
            
            
            if exportFigure == 0
                pause(0.01) 
                fprintf('At time: %f \n', time(t));
            end
            

            timeFrame = 1:t;

            for i = 1:N

                xpos = reshape(data(1,i,timeFrame), [length(timeFrame) 1]);
                ypos = reshape(data(2,i,timeFrame), [length(timeFrame) 1]);

                %
                % Remove the old plot.
                %
                if oldPlots(2*i) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    delete(oldPlots(2*i));
                    delete(oldPlots(2*i-1));
                end

                oldPlots(2*i-1) = plot(xpos, ypos, [colors{i} '--']);
                oldPlots(2*i) = plot(xpos(end), ypos(end), [colors{i} 'o']);
                %oldPlot = plot(xpos, ypos, '.');

                title(sprintf('Agent simulation at time: %.2fs \nUse keys "s" to stop, "r" to reset', time(t)));

                % Plot last position as a circle
                %plot(xpos(end), ypos(end), [colors{i} 'o']);


            end


            % Plot barycenter
            % Plot barycenter
            baryX = 1/N*reshape(sum(data(1,:,timeFrame), 2), [length(timeFrame) 1]);
            baryY = 1/N*reshape(sum(data(2,:,timeFrame), 2), [length(timeFrame) 1]);


            % Remove old plot
            if oldBaryPlot(1) ~= 0
                delete(oldBaryPlot(1))
                delete(oldBaryPlot(2))
            end

            %oldBaryPlot(1) = plot(baryX, baryY, 'k-.');
            %oldBaryPlot(2) = plot(baryX(end), baryY(end), 'kx');

            drawnow;


            %
            % Export?
            %
            if exportFigure
               % Check timeslot
               if  exportCounter <= length(exportTimeSlots) && time(t) >= exportTimeSlots(exportCounter)
                   
                   % Check if we should plot extras
                   if length(exportDoExtra) >= exportCounter
                      this_doExtra = exportDoExtra(exportCounter);
                   else
                       this_doExtra = 0;
                   end
                   
                   % Plot extra
                   if this_doExtra
                       plotExtra();
                   end
                   
                   
                   %path = 'export/';
                   
                   
                   % Export full picture
                   %exportPlot(fig, sprintf('%s_%d', exportFigureName,exportCounter), 'full');
                   
                   % Export small picture
                   %exportPlot(fig, sprintf('%s_%d_small', exportFigureName,exportCounter), 0.5);
                   
                   % Export small picture of where the agents are
                   oldAxis = axis;
                   
                   % "zoom" to the current positions
                   xpos = data(1,:,currentIndex);
                   ypos = data(2,:,currentIndex);
                   
                   %xlim([ratio*min([xpos]-0.1) ratio*max([xpos min(xpos)+2])]);
                   %ylim([ratio*min([ypos]-0.1) ratio*max([ypos min(ypos)+3])]);
                   
                   x = [min(xpos) max(xpos)];
                   y = [min(ypos) max(ypos)];
                   
                   xlim([min([x(1) mean(x)-exportMinBox(1)/2]), ...
                       max([x(2) mean(x)+exportMinBox(1)/2])]);
                   
                   ylim([min([y(1) mean(y)-exportMinBox(2)/2]), ...
                       max([y(2) mean(y)+exportMinBox(2)/2])]);
                   
                   %xlim([min(xpos)-exportMinBox(1)/6 max([xpos min(xpos)+exportMinBox(1)])]);
                   %ylim([min(ypos)-exportMinBox(2)/6 max([ypos min(ypos)+exportMinBox(2)])]);
                   
                   %exportPlot(fig, sprintf('%s_%d_small_zoom', exportFigureName,exportCounter), 0.5);
                   %exportPlot(fig, sprintf('%s_%d_zoom', exportFigureName,exportCounter), 'full');

                   %hgexport(fig, sprintf('%s%s%g.eps', path, exportFigureName, exportCounter));
                   %input('Press enter', 's')
                   
                   
                   % Restore old axis
                   axis(oldAxis);
                   
                   
                   % delete extra
                   if this_doExtra
                        if any(ishandle(oldExtraPlot))
                            delete(oldExtraPlot);
                            oldExtraPlot = [];
                        end
                   end
                   

                   % Increment the counter
                   exportCounter = exportCounter + 1;
               end

            end % end export
            
            % Testing some movie export
            %oldPos = get(fig, 'Position');            
            %set(fig, 'Position', [oldPos(1:2) 720 609]);     
            %M(currentIndex) = getframe; 
            %set(fig, 'Position', oldPos);
            
        end
        
        % done plotting 
        statusRun = 0;
        
    end % end function
% Plot barycenter
baryX = 1/N*reshape(sum(data(1,:,:), 2), [size(data, 3) 1]);
baryY = 1/N*reshape(sum(data(2,:,:), 2), [size(data, 3) 1]);

dataLength = size(data, 3);
lookupVector = [1:ceil(dataLength/10):dataLength];

%plot(baryX(lookupVector), baryY(lookupVector), 'rx');

function plotExtra
    if any(ishandle(oldExtraPlot))
        delete(oldExtraPlot);
        oldExtraPlot = [];
    end
    
    % Plot a circle around each agent, with the correct d
    
    k = 0:0.01:2*pi;
    
    for agent = 1:N
       pos = data(:,agent,currentIndex);
       r = CTRLconfig.collision_d;
       
       oldExtraPlot(end+1) = plot(pos(1) + r*cos(k), pos(2) + r*sin(k));
       
       % Plot velocity vector
       v                        = log.get('v_agent').Values.Data(:,agent,currentIndex);
       v_collision              = log.get('v_collision').Values.Data(:,agent,currentIndex);
       v_obstacle               = log.get('v_obst').Values.Data(:,agent,currentIndex);
       v_formation              = log.get('v_formation').Values.Data(:,agent,currentIndex);
       v_formation_collision    = log.get('v_formation_collision').Values.Data(:,agent,currentIndex);
       
       
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v, 'b--', 'v_d'));
       %oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_collision, 'g--', ''));
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_obstacle, 'k--'));
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_formation, 'r--', 'u'));
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_formation_collision, 'r-'));
       
       % Display information in the commandline
       fprintf('Agent: %d \n', agent);
       
       if log.get('collision_active').Values.Data(currentIndex, agent) == 1
           fprintf('\tCollision active \n');
       end
       if log.get('obstacle_active').Values.Data(currentIndex, agent) == 1
           fprintf('\tObstacle active \n');
       end
       
    end
    
    
    
end



function keyPress(src, eventData)
    fprintf('Pressed key: %s \n', eventData.Key);
    
    if eventData.Key == 's'
        statusRun = ~statusRun;
        
        % set correct stop time again (always full)
        stopIndex = stopIndex_full;
        
        % if starting
        if statusRun == 1
            continuePlot();
        else
           plotExtra(); 
        end
        
    end
    
    if eventData.Key == 'r'
        if statusRun == 0
            startIndex = 1;
            statusRun = 1;
            continuePlot();
        end
    end
    
    switch eventData.Key
        case 'd' % step
            if statusRun == 0
                startIndex = currentIndex;
                stopIndex = currentIndex + stepIndex;
                statusRun = 1;
                continuePlot();
                plotExtra();
            end
    end
end




end

function out = vector_append(v, new)
    out = zeros(1, length(new));
    for i = 1:length(new)
        out(i) = new(i);
    end
    
    out = [v out];
end

function handle = plotVector(pos, v, style)
%Plots the vector from the pos
% Does magic
    
    % find vector angle
    angle = atan2(v(2), v(1));
    
    % Get rotation matrix
    R = @(a)( [cos(a) -sin(a); sin(a) cos(a)]);
    
    % Length of vector
    length = norm(v);
    
    % Calculate the vector
    vector = R(angle) * [length; 0];
    
    % Line positions
    lineX = [pos(1) pos(1) + vector(1)];
    lineY = [pos(2) pos(2) + vector(2)];
    
    % Plot the line    
    handle = line(lineX, lineY);
    
    % Set style
    t = regexp(style, '(?<color>\w*)', 'names');
    color = t.color;
    t = regexp(style, '(?<linestyle>\W*)', 'names');
    linestyle = t.linestyle;
    
    set(handle, 'Color', color);
    set(handle, 'LineStyle', linestyle);
    
    
end


function handle = plotVectorWithAnnotation(pos, v, style, theText)
    % ARROW SETUP
    tip = 0.3;
    scale = 0.03;
    arrow = [0 0;
                -tip -1;
                0 -0.7;
                tip -1;
                0 0;]';   
    
    % find vector angle
    angle = atan2(v(2), v(1));
    
    % Get rotation matrix
    R = @(a)( [cos(a) -sin(a); sin(a) cos(a)]);
    
    % Length of vector
    length = 0.2;
    

    
    
    
    % If ca 0, do not plot arrow
    if norm(v) < 0.01
        handle = [];
        return;
    end
    
    % Plot the actual line
    handle = plotVector(pos, length*v/norm(v), style);
    
    % Calculate the vector
    vector = R(angle) * [length; 0];           
    stopPos = pos + vector;

   
    % Plot the arrowhead to Stop          
    target = R(angle+pi/2+pi)*scale*arrow;
    handle(end+1) = plot(stopPos(1)+target(1,:),stopPos(2)+target(2,:),'k');
    handle(end+1) = patch(stopPos(1)+target(1,:), stopPos(2)+target(2,:), 'k');
    
    % if plot text
    if nargin > 3
       tx = mean([pos(1) stopPos(1)]);
       ty = mean([pos(2) stopPos(2)]);
       
       % Augment a bit
       %tx = tx + 0.05*sign(tx);
       ty = ty + 0.03*sign(ty);
       
       handle(end+1) = text(tx, ty, theText);
    end

end




