%% setup
addpath('../../libraries');
p = 2; % Planar

N = 5; % 5 robots



D = connectAll(5);

% Obstacles
static_obstacles = [6 1; 10 -0.5;]';


% Initial Positions
initial_positions = [1 0.5; 0 0.5; 1 2; -1 -1; -1 0]';


% Desired Formation
desired_formation = [0 0; 2 2; 0 2; 2 0; 1 1]';
desired_formation_move    = [0 0; -1 1; -2 2; -2 -2; -1 -1]'./4;
desired_formation_search  = [0 0; 0 1; 0 2; 0 -2; 0 -1]';


angles = linspace(pi, -2*pi*4/5 + pi, 5);
r = 0.5;

desired_formation_contain = [r*cos(angles); r*sin(angles)];




% Desired barycenter
desired_barycenter_position = [4 1]';


desired_leader_pos_move = [15 0]';
desired_leader_pos_search = [18 0]';
desired_leader_pos_contain = [17.5 0]';


CTRLconfig.p = p;
CTRLconfig.N = N;
CTRLconfig.obstacle_d = 0.5;
CTRLconfig.collision_d = 0.2; 
CTRLconfig.debug = 0;
CTRLconfig.enable_guidance = 0;


 figure(2); clf
plotGraph(D);


%% Run



sim comoro_mission

%% Plotting

exportStruct.doExport = 0;
exportStruct.name = 'mission';
exportStruct.timeSlots = [0, 2 10 17 21 28 31 35 40];
exportStruct.doExtra = [0 0];
exportStruct.minBox = [3 3];

dataStruct.position = position.Data;
dataStruct.static_obstacles = static_obstacles;
dataStruct.time = position.Time;
dataStruct.endTime = position.Time(end);

M = plotSimulation(dataStruct, CTRLconfig, log, figure(1), exportStruct);
%%
%% Plot agent 1' velocity
figure(2); clf
data = velocity.Data;


time = position.Time;

agent = 1;

subplot(1,2,1)
plot(time, reshape(data(1,agent,:), [size(data, 3) 1]))
title('Agent 1 - velocity in x-direction');
ylabel('v_x');
xlabel('time [s]');
xlim([0 time(end)]);

subplot(1,2,2)
plot(time, reshape(data(2,agent,:), [size(data, 3) 1]))
title('Agent 1 - velocity in y-direction');
ylabel('v_y');
xlabel('time [s]');
xlim([0 time(end)]);



