/*
 * HardwareSerial.cpp
 *
 *  Created on: May 15, 2013
 *      Author: krisklau
 */

#include "HardwareSerial.h"

#include <sys/select.h>
#include <sys/time.h>
#include <errno.h>


#include <iostream>

using std::cerr;
using std::endl;
using std::endl;

HardwareSerial::HardwareSerial() {
	this->is_configured = false;
	this->is_opened = false;
	this->fd = 0;
}


int HardwareSerial::configure(UARTChannelType channel) {
	// TODO Auto-generated constructor stub
	this->channel = channel;

		std::string rx_filename = string();
		std::string tx_filename = string();
		std::string rx_mode = "0";
		std::string tx_mode = "0";

		// See which channel the input channel is
		// and set up the configuration file thereafter
		switch(this->channel)
		{
			case UART_1:
				this->port = "/dev/ttyO1";
				rx_filename = "/sys/kernel/debug/omap_mux/uart1_rxd";
				tx_filename = "/sys/kernel/debug/omap_mux/uart1_txd";
				rx_mode = "20";
				tx_mode = "0";
				break;
			case UART_2:
				this->port = "/dev/ttyO2";
				rx_filename = "/sys/kernel/debug/omap_mux/spi0_sclk";
				tx_filename = "/sys/kernel/debug/omap_mux/spi0_d0";
				rx_mode = "21";
				tx_mode = "1";
				break;
			case UART_4:
				this->port = "/dev/ttyO4";
				rx_filename = "/sys/kernel/debug/omap_mux/gpmc_wait0";
				tx_filename = "/sys/kernel/debug/omap_mux/gpmc_wpn";
				rx_mode = "26";
				tx_mode = "6";
				break;
			case UART_5:
				this->port = "/dev/ttyO5";
				rx_filename = "/sys/kernel/debug/omap_mux/lcd_data9";
				tx_filename = "/sys/kernel/debug/omap_mux/lcd_data8";
				rx_mode = "24";
				tx_mode = "4";
				break;
			default:
				cout << "uart_channel: configuration failed - uart " << this->channel << " does not exist" << endl;
				return -1;
		}

		FILE* uart_handler = NULL;

		// Configure the RX pin
		if((uart_handler = fopen(rx_filename.c_str(), "r+")) != NULL) {
			fwrite(rx_mode.c_str(), sizeof(char), rx_mode.size(), uart_handler);
			fclose(uart_handler);
		}
		else {
			cout << "uart_channel: unable to configure RX port" << endl;
			return -1;
		}

		// Configure the TX pin
		if((uart_handler = fopen(tx_filename.c_str(), "r+")) != NULL) {
			fwrite(tx_mode.c_str(), sizeof(char), tx_mode.size(), uart_handler);
			fclose(uart_handler);
		}
		else {
			cout << "uart_channel: unable to configure TX port" << endl;
			return -1;
		}




		struct termios settings;

		// Open port
		int port_file = ::open(this->port.c_str(), O_RDWR);
		if(port_file == -1) {
			cout << "uart_channel: unable to open port - " << this->port << endl;
			return -1;
		}

		// Get the current settings
		tcgetattr(port_file, &settings);

		// Set the input/output speed
		cfsetispeed(&settings, B57600);
		cfsetospeed(&settings, B57600);

		// Make the channel canonial
		// and have end of file as EOT ASCII char
		//settings.c_lflag = ICANON;		// enable
		settings.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // disable
		settings.c_cc[VEOF] = 0x4;

		// Apply the new settings
		if(tcsetattr(port_file, TCSANOW, &settings) < 0) {
			cerr << "uart_channel: Unable to set configuration" << endl;
			return -1;
		}

		::close(port_file);
		// Finished configuration
		this->is_configured = true;
		cout << "uart_channel: configured channel " << this->channel << " to port " << this->port << endl;


		// Open the shit
		return this->open();

}

int HardwareSerial::open() {
	// Opens the shit
	if(!this->is_configured) {
		cout << "uart_channel: read error - not configured" << endl;
		return -1;
	}

	// Open the channel
	int port_file = ::open(this->port.c_str(), O_RDWR| O_FSYNC);
	if(port_file < 0) {
		cerr << "Error opening file for reading" << endl;
		return -1;
	}

	this->is_opened = true;
	this->fd = port_file;
	return 0;


}

bool HardwareSerial::available() {
	// OKaaaay.
	// This method must use select to check of we will block.
	fd_set descriptorSet;
	struct timeval tv;

	FD_ZERO(&descriptorSet);
	FD_SET(this->fd, &descriptorSet);

	// Setup both fields to zero, select will return at once.
	tv.tv_sec = 0;
	tv.tv_usec = 0;

	int retval = select(this->fd+1, &descriptorSet, NULL, NULL, &tv);
	if (retval == -1) {
		cerr << "Host got error in select. " << endl;
	}
	else if(retval) { // Positive, means we got a select statement
		if (FD_ISSET(this->fd, &descriptorSet)) { // Should be true
			return true;
		}
		else {
			cerr << "Something terrible has happened!" << endl;
		}
	}
	else { // timeout
		return false;
	}


	return false;
}

uint8_t HardwareSerial::read() {
	// Simply reads one byte and returns it!
	uint8_t output = 0;
	::read(this->fd, &output, 1);
	/*
	if( (int length = ::read(this->fd, &output, 1)) >= 0) {
		// Set the message
		return output;
	}*/

	return output;

}

void HardwareSerial::write(uint8_t byte) {

	int written = ::write(this->fd, &byte, 1);

	if(written < 0) {
		cerr << "uart_channel: Error writing to file." << endl;
		cerr << strerror(errno) << endl;
	}

}

void HardwareSerial::flush() {
	// Simply do nothing!
}


HardwareSerial Serial;


