/*
 * analog_laser.cpp
 *
 *  Created on: Mar 5, 2013
 *      Author: adamleon
 */

#include "analog_laser.h"

AnalogDevice::AnalogDevice(AnalogPin pin) {
	this->analogPath = "/sys/devices/platform/omap/tsc/ain";
	this->analogPath += '0' + (int)pin;

	cout << "Initialized path: " << this->analogPath << endl;
}

int AnalogDevice::read() {
	int ain_file = open(this->analogPath.c_str(), O_RDONLY);
	if(ain_file >= 0) {
		int length = 0;
		char s = 'a';
		if((length = ::read(ain_file, &s, 1024)) > 0) {
			close(ain_file);
			return atoi(&s);
		} else {
			cout << "Unable to read: " << strerror(errno) << endl;
		}
		close(ain_file);
	}
	else {
		cout << "Unable to open: " << strerror(errno) << endl;
	}
	return -1;
}


float AnalogLaser::getFromTable(int measurement) {
	float value = (float)measurement;

	for(int i = 0; i < TABLE_SIZE-1; i++ ) {
		if ( this->lookupTable[i][0] <= value && this->lookupTable[i+1][0] >= value ) {
	            float diffx = value - this->lookupTable[i][0];
	            float diffn = this->lookupTable[i+1][0] - this->lookupTable[i][0];

	            return this->lookupTable[i][1] + ( this->lookupTable[i+1][1] - this->lookupTable[i][1] ) * diffx / diffn;
	    }
	}

	return 80;
}

AnalogLaser::AnalogLaser(AnalogPin pin) : AnalogDevice(pin) {
	float table[10][2] = {
			{370, 100},
			{372, 90},
			{438, 80},
			{505, 70},
			{570, 60},
			{680, 50},
			{819, 40},
			{1011, 30},
			{1431, 20},
			{3560, 10}
	};

	for(int i = 0; i < 10; i++) {
		for(int j = 0; j < 2; j++) {
			this->lookupTable[i][j] = table[i][j];
		}
	}
}

float AnalogLaser::measureDistance() {
	int value = this->read();

	if(value == -1) {
		return -1;
	}

	return this->getFromTable(value);
}


