/*
 * camera_host.h
 *
 *  Created on: Feb 25, 2013
 *      Author: krisklau
 */

#ifndef CAMERA_HOST_H_
#define CAMERA_HOST_H_

/*
 *
 */


// Uses the datapoint class
#include "datapoint.h"

enum CAMERA_POINT_COLOR {CAM_NONE = -1, CAM_BLUE = 1, CAM_RED, CAM_GREEN, CAM_CYAN, CAM_YELLOW, CAM_MAGENTA};
enum CAMERA_POINT_TYPE {NONE = -1, NONE_0 = 0, ROBOT = 1, TARGET, OBSTACLE, OBSERVED_POS, MISSION_V};

typedef struct {
	float x;
	float y;
} point_t_;

class point_t {
public:
	float x;
	float y;
	//bool valid;
};

typedef struct {
	point_t pos;
	point_t aref;
	bool valid;
	CAMERA_POINT_TYPE type;
	int size; // Contains the number of such elements
} cam_msg_t_;

class cam_msg_t {
public:
	point_t pos;
	point_t aref;
	bool valid;
	CAMERA_POINT_TYPE type;
	int size; // Contains the number of such elements
};

typedef cam_msg_t robot_t;
typedef cam_msg_t obstacle_t;
typedef cam_msg_t target_t;
class velocity_t {
public:
	point_t vel;
	velocity_t(cam_msg_t& cam): vel(cam.pos) {};
	velocity_t() {};
};

cam_msg_t getEmptyCamMsg(void);

/*
typedef struct {
	CAMERA_POINT_COLOR color;
	CAMERA_POINT_COLOR nearestTo;
} cam_request_msg_t;
*/

typedef struct {
	CAMERA_POINT_TYPE type;
	int iId;
} cam_request_msg_t;


class CameraClient : public DataPointRequestClient {
public:
	CameraClient():DataPointRequestClient("/cameraData", sizeof(cam_msg_t), "/cameraRequest", sizeof(cam_request_msg_t)) {};
	int request(cam_msg_t* msg) { return DataPointRequestClient::request((void*) msg);};
	int request(cam_msg_t* msg, int iRobotId);
	int request(cam_msg_t* msg, CAMERA_POINT_TYPE type, int iRobotId = -1);
	int request(velocity_t* msg);

private:
	using DataPointRequestClient::request;
	int request(void* msg, void* rq_msg) {return DataPointRequestClient::request(msg, rq_msg);};
};

class CameraHost: public DataPointRequestHost {
public:
	CameraHost(): DataPointRequestHost("/cameraData", sizeof(cam_msg_t), "/cameraRequest", sizeof(cam_request_msg_t)) {};
	int send_data(cam_msg_t* msg) { return DataPointRequestHost::send_data((void*) msg);};
	int read_request(cam_request_msg_t* rqMsg) { return DataPointRequestHost::read_request((void*) rqMsg);};
};






#endif /* CAMERA_HOST_H_ */
