/*
 * datapoint.h
 *
 *  Created on: Feb 8, 2013
 *      Author: krisklau
 */

#ifndef DATAPOINT_H_
#define DATAPOINT_H_


#include <mqueue.h>
#include <iostream>
#include <string>

using std::string;

#define QUEUE_MAXMSG 10 // 10 is max hard-coded in most OS.

enum DATAPOINT_TYPE {DP_READ, DP_WRITE};
enum DP_REQUEST_TYPE {DPR_HOST, DPR_CLIENT};

/* This is now intended to be set by the application */
typedef struct {
	int request;
	int secondRequest;
} DP_request_msg;

/*
 *
 */
class DataPoint {
public:
	DataPoint(DATAPOINT_TYPE type, string queueName, int msgSize);
	int open(void);
	int close(void);
	mqd_t get_mqd(void);
	int get_fd(void);

	int send(void* msg);
	int recieve(void* msg);

	int numberOfMessages(void);
	int get_msgSize(void);
private:
	DATAPOINT_TYPE type;
	string queueName;
	int msgSize;


	mqd_t mqDescriptor;

};



class DataPointRequest {
protected:
	DataPointRequest(DATAPOINT_TYPE dataType, DATAPOINT_TYPE requestType , string queueName, int msgSize, string requestQueueName, int requestMsgSize = sizeof(DP_request_msg));


public:
	int open(void);
	int get_request_fd(void);
protected:
	int read_request(DP_request_msg* msg);
	int read_request(void* msg);
	int send_data(void* msg);
	void forceClose(void);
	int request(void* msg);
	int request(void* msg, DP_request_msg* rq_msg);
	int request(void* msg, void* rq_msg);

	void set_timeout(int timeout_us);
private:

	DataPoint dataPoint;
	DataPoint requestPoint;
	int timeout_us;
};

class DataPointRequestHost: public DataPointRequest {
public:
	DataPointRequestHost(string queueName, int msgSize, string requestQueueName, int requestMsgSize = sizeof(DP_request_msg));
	int read_request(DP_request_msg* msg){ return DataPointRequest::read_request(msg);};
	int read_request(void* msg) { return DataPointRequest::read_request(msg);};
	int send_data(void* msg) {return DataPointRequest::send_data(msg);};
};

class DataPointRequestClient: public DataPointRequest {
public:
	DataPointRequestClient(string queueName, int msgSize, string requestQueueName, int requestMsgSize = sizeof(DP_request_msg));
	int request(void* msg){return DataPointRequest::request(msg);};
	int request(void* msg, DP_request_msg* rq_msg) { return DataPointRequest::request(msg, rq_msg);};
	int request(void* msg, void* rq_msg) {return DataPointRequest::request(msg, rq_msg);};
	using DataPointRequest::set_timeout;
};
#endif /* DATAPOINT_H_ */
