#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <linux/input.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "gpio.h"
/*
 * GLOBAL FUNCTIONS
 */

int write_mux_file(char *path, char *value, int len)
{
	int res;
	FILE *file_handler = NULL;

	if((file_handler = fopen(path, "ab")) == NULL) return 0;

	res = fwrite(value, sizeof(char), len, file_handler);
	fclose(file_handler);
	return res;
}

int gpio_export(int gpio)
{
	char buf[30];
	int fd;
	int rc;
	struct stat st;

	sprintf(buf, "/sys/class/gpio/gpio%i", gpio);

	if(stat(buf, &st) != 0)
	{
		// open
		sprintf(buf, "%i", gpio);
		fd = open("/sys/class/gpio/export", O_WRONLY);

		rc = write(fd, buf, strlen(buf));

		close(fd);

		return rc;
	}
	return 1;
}

int gpio_set_direction(int gpio, int dir)
{
	char buf[30];
	int fd;
	int rc;

	// open
	sprintf(buf, "/sys/class/gpio/gpio%i/direction", gpio);
	fd = open(buf, O_WRONLY);

	rc = -1;
	if(dir == DIR_IN) rc = write(fd, TEXT_DIR_IN, strlen(TEXT_DIR_IN));
	if(dir == DIR_OUT) rc = write(fd, TEXT_DIR_OUT, strlen(TEXT_DIR_OUT));

	close(fd);

	return rc;
}

int gpio_set_edge(int gpio, int edge)
{
	char buf[30];
	int fd;
	int rc;

	// open
	sprintf(buf, "/sys/class/gpio/gpio%i/edge", gpio);
	fd = open(buf, O_WRONLY);

	if(edge == EDGE_NONE) rc = write(fd, TEXT_EDGE_NONE, strlen(TEXT_EDGE_NONE));
	if(edge == EDGE_RISING) rc = write(fd, TEXT_EDGE_RISING, strlen(TEXT_EDGE_RISING));
	if(edge == EDGE_FALLING) rc = write(fd, TEXT_EDGE_FALLING, strlen(TEXT_EDGE_FALLING));
	if(edge == EDGE_BOTH) rc = write(fd, TEXT_EDGE_BOTH, strlen(TEXT_EDGE_BOTH));

	close(fd);

	return rc;
}

int gpio_open_read(int gpio)
{
	char buf[30];

	sprintf(buf, "/sys/class/gpio/gpio%i/value", gpio);
	return open(buf, O_RDONLY);
}

int gpio_open_write(int gpio)
{
	char buf[30];

	sprintf(buf, "/sys/class/gpio/gpio%i/value", gpio);
	return open(buf, O_WRONLY);
}

int gpio_read_int(int gpio)
{
	int fd = gpio_open_read(gpio);
	char buf[10];
	read(fd, buf, 10);
	close(fd);
	return atoi(buf);
}
