/*
 * i2cdevice.h
 *
 *  Created on: Feb 4, 2013
 *      Author: krisklau
 */

#ifndef I2CDEVICE_H_
#define I2CDEVICE_H_

enum I2C_LINE {I2C_1 = 1, I2C_2, I2C_3};

#ifndef PI
#define PI 3.14159265359
#endif

#ifndef RAD2DEG
#define RAD2DEG 57.2957795131
#endif

#ifndef DEG2RAD
#define DEG2RAD 0.01745329251
#endif


/*
 *
 */
class I2CDevice {
public:
	I2CDevice(I2C_LINE line, char address);
	int writeRegister(char registerAddress, char* data, int length);
	int writeRegister(char registerAddress, char data);

	int readRegister(char address, char* buffer, int length);

private:
	int line;
	char address;
};

class Magnetometer: public I2CDevice {
public:
	Magnetometer(char address);
	virtual int read(void) = 0;
	int setBias(int x, int y, int z);

	int calibrationStart(void);
	int calibrationStop(void);

	int getX(void) {return this->x;};
	int getY(void) {return this->y;};
	int getZ(void) {return this->z;};

	float getAngle(void); // gets angle
	int getAngleDeg_int(void);

protected:
	// Holds the current readings
	int x;
	int y;
	int z;

	// Bias addition.
	// This gets added to the mix.
	int bias_x;
	int bias_y;
	int bias_z;

	float scale_x;
	float scale_y;
	float scale_z;
protected:
	// Used for internal calibration
	int allMinMax[6];
	bool isCalibrating;

};

class HMC5883L : public Magnetometer {
public:
	HMC5883L();
	int read(void);
};

#endif /* I2CDEVICE_H_ */
