
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

#include "pwm.h"
#include "gpio.h"
#include "error.h"

static int write_file(char *file, char *buf, int len);

void pwm_init(int pwm, int frequency)
{
	char char_frequency[10];
	sprintf(char_frequency, "%i", frequency);

	if(pwm == PWM0)
	{
		// PWM0A
		// set pin mux
		if(!write_mux_file(PWM0A_MUX_FILE, PWM0A_MUX_VALUE, 1)) exit_error("PWM0A mux");
		// request
		if(!write_file(PWM0A_REQ, "1", 1)) exit_error("PWM0A request");
		// run
		if(!write_file(PWM0A_RUN, "1", 1)) exit_error("PWM0A run");
		// set frequency
		if(!write_file(PWM0A_FREQ, char_frequency, strlen(char_frequency))) exit_error("PWM0A frequency");
		// set duty to zero
		if(!pwm_set_percent(PWM0A, 0)) exit_error("PWM0A set value");
		
		// PWM0B
		// set pin mux
		if(!write_mux_file(PWM0B_MUX_FILE, PWM0B_MUX_VALUE, 1)) exit_error("PWM0B mux");
		// request
		if(!write_file(PWM0B_REQ, "1", 1)) exit_error("PWM0B request");
		// run
		if(!write_file(PWM0B_RUN, "1", 1)) exit_error("PWM0B run");
		// set frequency
		if(!write_file(PWM0B_FREQ, char_frequency, strlen(char_frequency))) exit_error("PWM0B frequency");
		// set duty to zero
		if(!pwm_set_percent(PWM0B, 0)) exit_error("PWM0B set value");
	}

	if(pwm == PWM1)
	{
		// PWM1A
		// set pin mux
		if(!write_mux_file(PWM1A_MUX_FILE, PWM1A_MUX_VALUE, 1)) exit_error("PWM1A mux");
		// request
		if(!write_file(PWM1A_REQ, "1", 1)) exit_error("PWM1A request");
		// run
		if(!write_file(PWM1A_RUN, "1", 1)) exit_error("PWM1A run");
		// set frequency
		if(!write_file(PWM1A_FREQ, char_frequency, strlen(char_frequency))) exit_error("PWM1A frequency");
		// set duty to zero
		if(!pwm_set_percent(PWM1A, 0)) exit_error("PWM1A set value");
		
		// PWM0B
		// set pin mux
		if(!write_mux_file(PWM1B_MUX_FILE, PWM1B_MUX_VALUE, 1)) exit_error("PWM1B mux");
		// request
		if(!write_file(PWM1B_REQ, "1", 1)) exit_error("PWM1B request");
		// run
		if(!write_file(PWM1B_RUN, "1", 1)) exit_error("PWM1B run");
		// set frequency
		if(!write_file(PWM1B_FREQ, char_frequency, strlen(char_frequency))) exit_error("PWM1B frequency");
		// set duty to zero
		if(!pwm_set_percent(PWM1B, 0)) exit_error("PWM1B set value");
	}
}

int pwm_set_ns(int pwm_ch, long value)
{
	char char_ns[10];

	sprintf(char_ns, "%li", value);

	if(pwm_ch == PWM0A) return write_file(PWM0A_NS, char_ns, strlen(char_ns));
	if(pwm_ch == PWM0B) return write_file(PWM0B_NS, char_ns, strlen(char_ns));
	if(pwm_ch == PWM1A) return write_file(PWM1A_NS, char_ns, strlen(char_ns));
	if(pwm_ch == PWM1B)
	{
		//printf("file: %s\n", PWM1B_NS);
		//printf("string: %s\n", char_ns);

		if( (write_file(PWM1B_NS, char_ns, strlen(char_ns))) <= 0 ) exit_error("PWM");
		return 1;

	}
	
	return 0;
}

int pwm_set_percent(int pwm_ch, int value)
{
	char char_percent[10];
	if(value < 0) value = 0;
	if(value > 100) value = 100;

	sprintf(char_percent, "%i", value);

	if(pwm_ch == PWM0A) return write_file(PWM0A_PERCENT, char_percent, strlen(char_percent));
	if(pwm_ch == PWM0B) return write_file(PWM0B_PERCENT, char_percent, strlen(char_percent));
	if(pwm_ch == PWM1A) return write_file(PWM1A_PERCENT, char_percent, strlen(char_percent));
	if(pwm_ch == PWM1B) return write_file(PWM1B_PERCENT, char_percent, strlen(char_percent));	
	
	return 0;
}


int write_file(char *path, char *value, int len)
{
	int res;
	FILE *file_handler = NULL;

	if((file_handler = fopen(path, "wb")) == NULL) return 0;

	res = fwrite(value, sizeof(char), len, file_handler);
	fclose(file_handler);
	return res;
}

/*
 * Stops the PWM module
 */
int pwm_stop(int pwm_ch) {

	switch(pwm_ch) {
	case PWM0A:
		if(!write_file(PWM0A_RUN, "0", 1)) exit_error("PWM0A run");
		break;
	case PWM0B:
		if(!write_file(PWM0B_RUN, "0", 1)) exit_error("PWM0B run");
		break;
	case PWM1A:
		if(!write_file(PWM1A_RUN, "0", 1)) exit_error("PWM1A run");
		break;
	case PWM1B:
		if(!write_file(PWM1B_RUN, "0", 1)) exit_error("PWM1B run");
		break;

	}
	return 0;
}

/*
 * Stops the PWM module
 */
int pwm_start(int pwm_ch) {

	switch(pwm_ch) {
	case PWM0A:
		if(!write_file(PWM0A_RUN, "1", 1)) exit_error("PWM0A run");
		break;
	case PWM0B:
		if(!write_file(PWM0B_RUN, "1", 1)) exit_error("PWM0B run");
		break;
	case PWM1A:
		if(!write_file(PWM1A_RUN, "1", 1)) exit_error("PWM1A run");
		break;
	case PWM1B:
		if(!write_file(PWM1B_RUN, "1", 1)) exit_error("PWM1B run");
		break;

	}
	return 0;
}
