/*
 * servo.cpp
 *
 *  Created on: Feb 4, 2013
 *      Author: krisklau
 */

#include "servo.h"
#include "pwm.h"
#include <stdio.h>


// FOR DEBUG
#include <iostream>
using std::cout;
using std::endl;

long map(long x, long in_min, long in_max, long out_min, long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}
long map(double x, long in_min, long in_max, long out_min, long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

Servo::Servo(SERVO_NAME servo) {
	// TODO Auto-generated constructor stub
	switch(servo) {
	case SERVO_0A:
		this->pwm = PWM0;
		this->pwm_ch = PWM0A;
		break;
	case SERVO_0B:
			this->pwm = PWM0;
			this->pwm_ch = PWM0B;
			break;
	case SERVO_1A:
			this->pwm = PWM1;
			this->pwm_ch = PWM1A;
			break;
	case SERVO_1B:
			this->pwm = PWM1;
			this->pwm_ch = PWM1B;
			break;
	default:
		fprintf(stderr, "Invalid servo given. Reverting to default, no EXIT. ");
		this->pwm = PWM0;
		this->pwm_ch = PWM0A;
		break;
	}
}


int Servo::init() {


	pwm_init(this->pwm, 50);
	this->start();
	return 0;


}
int Servo::start() {
	return pwm_start(this->pwm_ch);
}
int Servo::write(double value) {
	// check value
	if (value < 0) value = 0;
	if (value > 180) value = 180;

	long lvalue = map(value, 0, 180, 544E3, 2400E3); // numbers from arduino servo library

	int out =  pwm_set_ns(this->pwm_ch, lvalue);



	return out;
}

int Servo::write(int value) {
	// check value
	if (value < 0) value = 0;
	if (value > 180) value = 180;

	// Convert to 1-2 range
	double fvalue = 1.0 + value/180.0;

	// Set duty cycle
	//cout << "Set value: " << value << " Writing: " << fvalue*1E6 << endl;
	//this->stop();
	long lvalue = map(long(value), 0, 180, 544E3, 2400E3);
	//cout << "New: " << lvalue << endl;
	int out =  pwm_set_ns(this->pwm_ch, lvalue); // 1E6*fvalue);
	//this->start();
	return out;
}

int Servo::stop() {
	return pwm_stop(this->pwm_ch);
}
