/*
 * app_cameraClient.cpp
 *
 *  Created on: Feb 25, 2013
 *      Author: krisklau
 */




/* This stuff will read much from the pc
 * and then make it availible to a controller.
 */



#include <iostream>

#include <sys/select.h>
#include <sys/time.h>

#include <stdlib.h>

#include "camera_client.h"

#include "uart_channel.h"

#include <stdarg.h>

#include "pointContainer.h"
#include "StringController.h"
#include "json/json.h"

#include <string>

/* Defining the different colors to use */

using namespace std;

/*
void GetMatches ( const char * str, const char * format, ... )
{
  va_list args;
  va_start (args, format);
  vsscanf (str, format, args);
  va_end (args);
}
*/


int main(int argc, char** argv) {



	// Set up array of six cam points
	//PointContainer container;
	RobotContainer container;
	RobotContainer observedContainer;
	RobotContainer missionVelocityContainer;
	ObstacleContainer obstacleContainer;
	cam_msg_t target = getEmptyCamMsg();



	cout << "Starting.." << endl;


	fd_set descriptorSet;
	struct timeval tv;

	CameraHost host;
	host.open();



	// Setup Uart xbee communication
	UARTChannel uart;
	uart.configure(UART_2);

	// Set up communication to the udp module
	UdpCameraHost camera;
	Json::Reader reader;
	Json::Value jsonObject;

	// Open it up!
	uart.open();

	// Set up data
	cam_msg_t dataMsg;


	while(1) {

		FD_ZERO(&descriptorSet); 				// Init

		FD_SET(host.get_request_fd(), &descriptorSet); // Set
		FD_SET(uart.get_fd(), &descriptorSet);
		FD_SET(camera.get_fd(), &descriptorSet);

		// Setup 2 seconds timeout
		tv.tv_sec = 2;
		tv.tv_usec = 0;

		int retval = select(1024, &descriptorSet, NULL, NULL, &tv);

		if (retval == -1) {
			cerr << "Host got error in select. " << endl;
		}
		else if(retval) { // Positive, means we got a select statement
			if (FD_ISSET(host.get_request_fd(), &descriptorSet)) {
				cam_request_msg_t rqMsg;
				if(host.read_request(&rqMsg) < 0) {
					cerr << "Invalid request msg" << endl;
				}

				//cout << "Camera Got request: " << rqMsg.request << endl;
				cam_msg_t msgToSend;
				// Get message from container

				if(rqMsg.type == ROBOT) {
					msgToSend = container.get(rqMsg.iId);
					// Send
					host.send_data(&msgToSend);
				}
				else if(rqMsg.type == OBSERVED_POS) {
					msgToSend = observedContainer.get(rqMsg.iId);
					// Send
					host.send_data(&msgToSend);
				}
				else if(rqMsg.type == MISSION_V) {
					msgToSend = missionVelocityContainer.get(rqMsg.iId);
					//send
					host.send_data(&msgToSend);
				}
				else if(rqMsg.type == OBSTACLE) {
					msgToSend = obstacleContainer.get(rqMsg.iId);
					// Send
					host.send_data(&msgToSend);
				}
				else if(rqMsg.type == TARGET) {
					host.send_data(&target);
				}
				else if(rqMsg.type == NONE && rqMsg.iId == -1) {
					// Issued a reset
					obstacleContainer.clear();
					container.clear();
					target = getEmptyCamMsg();

					host.send_data(&target);
				}
				else {
					cerr << "Invalid request" << endl;
				}





			}
			else if(FD_ISSET(camera.get_fd(), &descriptorSet)) {
				// Got cameramessage form the udp server.
				// This message is a json packet.
				string message = camera.receive();

				if(!reader.parse(message, jsonObject)) {
					cerr << "invalid packet received" << endl;
				}
				else {
					/*
					cam_msg_t newMessage;
					newMessage.x = jsonObject.get("x", 0.0).asFloat();
					newMessage.y = jsonObject.get("y", 0.0).asFloat();
					newMessage.valid = true;

					// Set color
					_setColor(jsonObject.get("color", "r").asString().at(0), &newMessage);

					// Store
					container.update(newMessage);
					*/

					// Check valid packet
					string type = jsonObject.get("type", "invalid").asString();
					if(type.compare("position_update")==0) {

						cam_msg_t robot;
						Json::Value pos;


						pos = jsonObject.get("position", pos);

						robot.pos.x = pos.get("x", 0.0).asFloat();
						robot.pos.y = pos.get("y", 0.0).asFloat();




						int iRobotId = jsonObject.get("robot_id", 1).asInt();


						robot.valid = true;
						robot.type = OBSERVED_POS;

						//cout << "Camera got:  " << robot.pos.x << " - " << robot.pos.y << endl;
						//cout << "Aref: " << robot.aref.x << " - " << robot.aref.y << endl;
						// store
						observedContainer.update(iRobotId, robot);
					}
					else if(type == "common_v") {

						// Somewhat a hack, store the velocity in a position storage
						// But sometimes, just get the shit done.
						cam_msg_t robot;
						Json::Value vel;

						vel = jsonObject.get("velocity", vel);
						robot.pos.x = vel.get("x", 0.0).asFloat();
						robot.pos.y = vel.get("y", 0.0).asFloat();

						// For now, just store the freaking thing as id = 1;
						robot.valid = true;
						robot.type = MISSION_V;

						missionVelocityContainer.update(1, robot);
					}


					else if(type.compare("camera")==0) {
						// got something raw from the camera

						string object_type = jsonObject.get("object_type", "invalid").asString();

						if(object_type.compare("target") == 0) {
							Json::Value pos;

							pos = jsonObject.get("position", pos);

							target.valid = true;
							target.pos.x = pos.get("x", -10).asFloat();
							target.pos.y = pos.get("y", -10).asFloat();
							target.type = TARGET;
							target.size = 1;
						}
						else if(object_type.compare("obstacle") == 0) {
							// obstacle


							// Got obstacles.
							// Clear old vector container, loop in new ones.
							obstacleContainer.clear();

							Json::Value obstacles = jsonObject.get("obstacles", "");
							obstacleContainer.add(obstacles);



						}
						else if(object_type.compare("robot") == 0){

							cam_msg_t robot;
							Json::Value pos;
							Json::Value aref;

							pos = jsonObject.get("position", pos);
							aref = jsonObject.get("aref", aref);
							robot.pos.x = pos.get("x", 0.0).asFloat();
							robot.pos.y = pos.get("y", 0.0).asFloat();

							robot.aref.x = aref.get("x", 0.0).asFloat();
							robot.aref.y = aref.get("y", 0.0).asFloat();

							int iRobotId = jsonObject.get("robot_id", 1).asInt();


							robot.valid = true;
							robot.valid = pos.get("valid", true).asBool();
							robot.type = ROBOT;

							//cout << "Camera got:  " << robot.pos.x << " - " << robot.pos.y << endl;
							//cout << "Aref: " << robot.aref.x << " - " << robot.aref.y << endl;
							// store
							container.update(iRobotId, robot);
						}

					}

				}


			}
			else if(FD_ISSET(uart.get_fd(), &descriptorSet)) {
				//cout << "Got camera data" << endl;


				string msg;

				if(uart.read(msg) < 0) {
					cerr << "error reading. " << endl;
					cerr << strerror(errno) << endl;
				}
				/*
				// Use split on the stuffs!
				float x = 0, y = 0;
				char color;

				cam_msg_t newMessage;
				newMessage.valid = true;
				cout << "Got something: " << msg << endl;
				GetMatches(msg.c_str(), "%c:%f:%f", &color, &x, &y);

				// Set coordinates
				newMessage.x = x;
				newMessage.y = y;

				// Set color
				switch(color) {
				case 'g':
					newMessage.color = CAM_GREEN;
					break;
				case 'y':
					newMessage.color = CAM_YELLOW;
					break;
				case 'b':
					newMessage.color = CAM_BLUE;
					break;
				case 'c':
					newMessage.color = CAM_CYAN;
					break;
				case 'm':
					newMessage.color = CAM_MAGENTA;
					break;
				case 'r':
					newMessage.color = CAM_RED;
					break;
				default:
					cerr << "Invalid color received! Ignoring.." << endl;
					break;
				}

				// Store
				container.update(newMessage);
				*/

			}
			else {
				cerr << "Invalid select out return." << endl;
			}

		}
		else { // timeout
			cerr << "Camera timeout." << endl;
		}

	}


	return 0;

}

