/*
 * main.cpp
 *
 *  Created on: Feb 13, 2013
 *      Author: krisklau
 */



#include "timer.h"
#include "servo_controller.h"
#include "imudata.h"
#include "robot.h"
#include "i2cdevice.h"
#include <math.h>
#include "uart_channel.h"
#include "camera_client.h"
#include "logger.h"

#include "servo.h"

#include "Subsystem.h"

#include <iostream>
#include <vector>
#include <string>
#include <sstream>

#include <stdarg.h>
#include <stdio.h>

using namespace std;



std::vector<std::string> &split(const std::string &s, char delim, std::vector<std::string> &elems) {
    std::stringstream ss(s);
    std::string item;
    while(std::getline(ss, item, delim)) {
        elems.push_back(item);
    }
    return elems;
}


std::vector<std::string> split(const std::string &s, char delim) {
    std::vector<std::string> elems;
    return split(s, delim, elems);
}

void GetMatches ( const char * str, const char * format, ... )
{
  va_list args;
  va_start (args, format);
  vsscanf (str, format, args);
  va_end (args);
}

int main(int argc, char** argv) {

	// Set up periodic
	periodic_info timerInfo;

	// Set up servo connection
	RobotClient robot;

	// Set up xbee interface
	UARTChannel xbee;
	xbee.configure(UART_2);

	// Set up IMU connection
	IMUDataClient imu;
	imu.open();

	// Set up camera connection
	CameraClient camera;
	camera.open();

	// TMP: Magnetometer
	HMC5883L mag;

	// IMU data
	IMU_data_msg imuData;
	ServoMessage servoData;

	// Loggger
	Logger logger;
	logger.init();



	cout << "Starting loop. " << endl;

	//Robot robot;
	//robot.init();

	mag.read(); // First is rubbish?
	int counter = 0;
	mag.calibrationStart();


	// INIT matlab
	Subsystem_initialize();
	Kp = 50;
	// Start periodic timer




	make_periodic(1E5, &timerInfo);
	while(1) {
		logger.timestamp();
		//robot.send(0, -40, 40);
		string msg;
		//cout << "Attempting read: " << endl;
		//if(xbee.read(msg) < 0) {
		//	cerr << "error reading. " << endl;
		//	cerr << strerror(errno) << endl;
		//}
		//cout << "Done read." << endl;
		//xbee.write("Fra bb");

		// Use split on the stuffs!

		// Get from camera host
		cam_msg_t camMsg;
		camera.request(&camMsg, CAM_GREEN,CAM_YELLOW);

		float greenX = 0, greenY = 0, yellowX = 0, yellowY = 0;
		//GetMatches(msg.c_str(), "%f:%f;%f:%f", &greenX, &greenY, &yellowX, &yellowY);
		//cout << "Converted: " << greenX << " - " << greenY;
		//cout << endl << yellowX << " - " << yellowY << endl;

		greenX = camMsg.x;
		greenY = camMsg.y;

		yellowX = camMsg.aref_x;
		yellowY = camMsg.aref_y;


		float diffX = yellowX - greenX;
		float diffY = yellowY - greenY;

		Subsystem_U.mag.x = static_cast<int16_t>(diffX);
		Subsystem_U.mag.y = static_cast<int16_t>(diffY);
		logger.add("X", greenX);
		logger.add("Y", greenY);


		// Set camera input
		Subsystem_U.camera.pos.point[0] = greenX;
		Subsystem_U.camera.pos.point[1] = greenY;

		Subsystem_U.camera.aref.point[0] = yellowX;
		Subsystem_U.camera.aref.point[1] = yellowY;

		Subsystem_U.desired_pos.point[0] = 200.0;
		Subsystem_U.desired_pos.point[1] = 200.0;

		Subsystem_step();

		logger.addTable(logEntry, 10, "mlog");


		//cout << "Angle: " << angle*RAD2DEG << endl;

		// Cout output
		cout << "Matlab got angle: " << Subsystem_Y.heading << endl;
		cout << "Current pos: " << Subsystem_Y.current_pos[0] << " - " <<  Subsystem_Y.current_pos[1]<< endl;
		cout << "Aref: " << yellowX << " - " << yellowY << endl;
		// Send servos!


		// Add servos to log
		logger.addTable(Subsystem_Y.servo_out, 3, "servo");


		robot.send(Subsystem_Y.servo_out, 3);
		//robot.send(0,-7,7);
		cout << "Servo: " << Subsystem_Y.servo_out[0] << " - " << Subsystem_Y.servo_out[1] << " - " << Subsystem_Y.servo_out[2] << endl;



		logger.write();

		wait_period(&timerInfo);
	}


	while(1) {
		counter = counter + 100;
		counter = counter % 2000;
		// Set inputsl
		Subsystem_U.mag.x = 1000;
		Subsystem_U.mag.y = counter - 1000;

		cout << "Counter: " << counter << endl;

		Subsystem_step();

		// Cout output
		cout << "Matlab got angle: " << Subsystem_Y.heading << endl;
		// Send servos!
		robot.send(Subsystem_Y.servo_out[0], Subsystem_Y.servo_out[1], Subsystem_Y.servo_out[2]);

		// wait
		wait_period(&timerInfo);
	}


	while(1) {

			servoData.a = 0;
			servoData.b = 0;
			servoData.c = 0;


			robot.send(servoData);
			sleep(1);
			servoData.a = 20;
			servoData.b = 20;
			servoData.c = 20;


			robot.send(servoData);
			sleep(1);

			servoData.a = 0;
			servoData.b = 0;
			servoData.c = 0;


			robot.send(servoData);
			sleep(1);


			servoData.a = -20;
			servoData.b = -20;
			servoData.c = -20;


			robot.send(servoData);
			sleep(1);

		}



	while(1) {
		mag.read();
		cout << "Current angle: " << mag.getAngleDeg_int() << endl;
		cout << "Current X: " << mag.getX() << endl;
		cout << "Current Y: " << mag.getY() << endl;
		wait_period(&timerInfo);

		if(counter > 100) {
			mag.calibrationStop();
			cout << "Calib complete" << endl;
		}

		counter++;
	}




	// Loop
	while(1) {
		// Read IMU data
		if(imu.request(&imuData) < 0) {
			cerr << "ERROR in imu read" << endl;
		}
		else{
			// Send it to servo
			cout << "Data ok." << endl;
			servoData.a = imuData.data;
			robot.send(servoData);
		}

		wait_period(&timerInfo);
	}

	return 0;
}



