/*
 * main.cpp
 *
 *  Created on: Feb 13, 2013
 *      Author: krisklau
 */



#include "timer.h"

#include "camera_client.h"

#include "logger.h"

#include "servo_controller.h"

#include "particle_controller.h"

#include "Subsystem.h"
#include "Observer.h"

#include <math.h>
#include <iostream>
#include <vector>
#include <string>
#include <sstream>

#include <stdarg.h>
#include <stdio.h>

using namespace std;





int main(int argc, char** argv) {

	// Set up periodic
	periodic_info timerInfo;

	// Set up servo connection
	RobotClient robot;



	// Set up camera connection
	CameraClient camera;
	camera.open();

	// Set up connection to particle adam stuff
	ParticleClient laser;
	StateMessage laserMsg;


	// Loggger
	Logger logger;
	logger.init();



	cout << "Starting loop. " << endl;

	//Robot robot;
	//robot.init();



	// INIT matlab
	Subsystem_initialize();
	//Kp = 5;
	// Start periodic timer



	int counter = 0;
	make_periodic(1E5, &timerInfo);
	while(1) {
		/*
		 * INIT
		 */

		logger.timestamp();

		/*
		 * Request and set incomming sensor data
		 */

		cam_msg_t green;
		camera.request(&green, CAM_GREEN);

		cam_msg_t yellow;
		camera.request(&yellow, CAM_YELLOW, CAM_GREEN); // Yellow, nearest green

		cam_msg_t cyan;
		camera.request(&cyan, CAM_CYAN);

		cam_msg_t blue;
		camera.request(&blue, CAM_BLUE);


		logger.add("X", green.x);
		logger.add("Y", green.y);
		logger.add("pos_valid", green.valid);
		logger.add("aref_X", yellow.x);
		logger.add("aref_Y", yellow.y);
		logger.add("angle_valid", yellow.valid);

		logger.add("target_X", cyan.x);
		logger.add("target_Y", cyan.y);

		logger.add("obstacle_X", blue.x);
		logger.add("obstacle_Y", blue.y);
		logger.add("obstacle_valid", blue.valid);


		/*
		 * Set input to matlab
		 */
		Subsystem_U.camera.pos.point[0] = green.x;
		Subsystem_U.camera.pos.point[1] = green.y;
		Subsystem_U.camera.pos.valid = green.valid;

		Subsystem_U.camera.aref.point[0] = yellow.x;
		Subsystem_U.camera.aref.point[1] = yellow.y;
		Subsystem_U.camera.aref.valid = yellow.valid;

		Subsystem_U.desired_pos.point[0] = cyan.x;
		Subsystem_U.desired_pos.point[1] = cyan.y;

		Subsystem_U.obstacle.point[0] = blue.x;
		Subsystem_U.obstacle.point[1] = blue.y;

		/*
		 * Run controller step
		 */

		Subsystem_step();

		/*
		 * Logging and printing
		 */



		// Add matlablog to logfile.
		logger.addTable(logEntry, 10, "mlog");

		// Add estimated position to logfile
		logger.addTable(Subsystem_Y.current_pos, 2, "observed_pos");

		// Add observed heading to log
		logger.add("heading", static_cast<double>(Subsystem_Y.heading));

		// Add servos to log
		logger.addTable(Subsystem_Y.servo_out, 3, "servo");

		// Send servodata to the servocontroller
		robot.send(Subsystem_Y.servo_out, 3);

		// Add measured obstacle position to log
		logger.addTable(Subsystem_U.obstacle.point, 2, "obstacle");


		// Cout output
		cout << "Matlab got angle: " << Subsystem_Y.heading << endl;
		cout << "Current pos: " << Subsystem_Y.current_pos[0] << " - " <<  Subsystem_Y.current_pos[1]<< endl;
		cout << "Aref: " << yellow.x << " - " << yellow.y << endl;
		cout << "Servo: " << Subsystem_Y.servo_out[0] << " - " << Subsystem_Y.servo_out[1] << " - " << Subsystem_Y.servo_out[2] << endl;
		cout << "Blue: " << Subsystem_U.obstacle.point[0] << " - " << Subsystem_U.obstacle.point[1] << endl;

		// Send data to laser
		laserMsg.x = float(Subsystem_Y.current_pos[0]);
		laserMsg.y = float(Subsystem_Y.current_pos[1]);
		laserMsg.heading = float((M_PI/180)*logEntry[0]);

		laser.send(laserMsg);

		logger.execTimestamp();

		// Actually do the writing

		logger.write();


		counter++;

		// Wait untill next timer event.
		wait_period(&timerInfo);
	}


	return 0;
}



