/*
 * main.cpp
 *
 *  Created on: Feb 13, 2013
 *      Author: krisklau
 */



#include "timer.h"
#include "servo_controller.h"
#include "imudata.h"
#include "robot.h"
#include "i2cdevice.h"
#include <math.h>
#include "uart_channel.h"
#include "camera_client.h"
#include "logger.h"

#include "servo.h"

#include "Subsystem.h"

#include <iostream>
#include <vector>
#include <string>
#include <sstream>

#include <stdarg.h>
#include <stdio.h>

using namespace std;





int main(int argc, char** argv) {

	// Set up periodic
	periodic_info timerInfo;

	// Set up servo connection
	RobotClient robot;



	// Set up camera connection
	CameraClient camera;
	camera.open();


	// Loggger
	Logger logger;
	logger.init();



	cout << "Starting loop. " << endl;

	//Robot robot;
	//robot.init();



	// INIT matlab
	Subsystem_initialize();
	Kp = 50;
	// Start periodic timer



	int counter = 0;
	make_periodic(1E5, &timerInfo);
	while(1) {
		/*
		 * INIT
		 */

		logger.timestamp();

		/*
		 * Request and set incomming sensor data
		 */

		cam_msg_t camMsg;
		camera.request(&camMsg, CAM_GREEN,CAM_YELLOW);

		float greenX = 0, greenY = 0, yellowX = 0, yellowY = 0;
		//GetMatches(msg.c_str(), "%f:%f;%f:%f", &greenX, &greenY, &yellowX, &yellowY);
		//cout << "Converted: " << greenX << " - " << greenY;
		//cout << endl << yellowX << " - " << yellowY << endl;

		greenX = camMsg.x;
		greenY = camMsg.y;

		yellowX = camMsg.aref_x;
		yellowY = camMsg.aref_y;


		logger.add("X", greenX);
		logger.add("Y", greenY);
		logger.add("pos_valid", camMsg.valid);
		logger.add("angle_valid", camMsg.aref_valid);


		// Set camera input
		Subsystem_U.camera.pos.point[0] = greenX;
		Subsystem_U.camera.pos.point[1] = greenY;

		Subsystem_U.camera.aref.point[0] = yellowX;
		Subsystem_U.camera.aref.point[1] = yellowY;

		Subsystem_U.desired_pos.point[0] = 200.0;
		Subsystem_U.desired_pos.point[1] = 200.0;

		/*
		 * Run controller step
		 */

		Subsystem_step();

		/*
		 * Logging and printing
		 */
		// Do my special servo fun
		if(counter < 50) {
			Subsystem_Y.servo_out[0] = -20;
			Subsystem_Y.servo_out[1] = 10;
			Subsystem_Y.servo_out[2] = 10;
		}
		else if(counter < 80) {
			Subsystem_Y.servo_out[0] = 0;
			Subsystem_Y.servo_out[1] = -20;
			Subsystem_Y.servo_out[2] = 20;

		}
		else if(counter < 130) {
			Subsystem_Y.servo_out[0] = 20;
			Subsystem_Y.servo_out[1] = -10;
			Subsystem_Y.servo_out[2] = -10;

		}
		else {
			cout << "done" << endl;
			Subsystem_Y.servo_out[0] = 0;
			Subsystem_Y.servo_out[1] = 0;
			Subsystem_Y.servo_out[2] = 0;
			return 1;
		}


		// Add matlablog to logfile.
		logger.addTable(logEntry, 10, "mlog");

		// Add servos to log
		logger.addTable(Subsystem_Y.servo_out, 3, "servo");

		// Send servodata to the servocontroller
		robot.send(Subsystem_Y.servo_out, 3);

		// Cout output
		cout << "Matlab got angle: " << Subsystem_Y.heading << endl;
		cout << "Current pos: " << Subsystem_Y.current_pos[0] << " - " <<  Subsystem_Y.current_pos[1]<< endl;
		cout << "Aref: " << yellowX << " - " << yellowY << endl;
		cout << "Servo: " << Subsystem_Y.servo_out[0] << " - " << Subsystem_Y.servo_out[1] << " - " << Subsystem_Y.servo_out[2] << endl;



		// Actually do the writing
		logger.write();

		counter++;

		// Wait ntill next timer event.
		wait_period(&timerInfo);
	}


	return 0;
}



