/*
 * main.cpp
 *
 *  Created on: Feb 13, 2013
 *      Author: krisklau
 */



#include "timer.h"

#include "camera_client.h"

#include "logger.h"

#include "servo_controller.h"

#include "particle_controller.h"

#include "commandParser.h"

#include "Controller.h"
#include "Observer.h"

#include <math.h>
#include <pthread.h>
#include <iostream>
#include <vector>
#include <string>
#include <sstream>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h> // atoi

#include "json/json.h"
#include "StringController.h"
#include "message_client.h"
#include "uart_channel.h"

using namespace std;
using Json::Value;


// Setup supersimple state machine

enum STATE {INIT, GO};


int main(int argc, char** argv) {
	bool keepProcessRunning = true;
	while(keepProcessRunning) { // Super main loop

	/* Parse arguments */
	if(argc < 2) {
		cout << "Invalid arguments given. Must supply robot number" << endl;
		return -1;
	}
	int nRobots = -1;
	Formation formation = Square;
	bool activateMission = false;
	int robotId = atoi(argv[1]);

	// If robot 1, must supply number of other robots
	bool autoStart = false;


	if(argc > 2 && robotId == 1) {
		autoStart = true;
		nRobots = atoi(argv[2]);
	}



	// Connection to the other robots via udp
	UdpClient comm;
	MessageClient messageClient;
	messageClient.open();
	messageClient.set_timeout(9E5);

	// If this is the master robot (that is, id = 1), prepare a broadcast start message
	if(autoStart) {
		Value msg;
		msg["type"] = "command";
		msg["command"] = "start";
		msg["nRobots"] = nRobots;
		msg["formation"] = "square";

		comm.send(msg.toStyledString());

	}
	else {
		// Wait for start command
		bool start = false;
		Json::Reader reader;
		while(!start) {

			Value jMsg = messageClient.requestJson();
			COMMAND command = getCommand(jMsg);
			if(command == START) {
				nRobots = jMsg.get("nRobots", 1).asInt();

				// only start if we need to
				if(nRobots < robotId)
					continue;

				formation = getFormation(jMsg);
				activateMission = getActivateMission(jMsg);
				start = true;
			}
			else if(command == KILL){
				cout << "Got stopped kill" << endl;
				keepProcessRunning = false;
				return 0;
				//cout << "Got: " << (int) command << endl;

			}



		}

	}


	/*
	 * Set Number of active robots
	 */

	runConfig.nRobots = nRobots;
	runConfig.activateLeaderV = activateMission;
	runConfig.forceObstacleDeactivate = true;
	runConfig.formation = formation;
	runConfig.formationRotation = 0;

	RunConfig config_default = runConfig;

	cout << "N agents: " << runConfig.nRobots << endl;

	// Set up runconfigs based on states
	STATE state = INIT;
	RunConfig config_init;

	config_init.nRobots = nRobots;
	config_init.activateLeaderV = false;
	config_init.forceObstacleDeactivate = true;
	config_init.formation = formation;
	config_init.formationRotation = 0;

	RunConfig config_go;

	config_go.nRobots = nRobots;
	config_go.activateLeaderV = activateMission;
	config_go.forceObstacleDeactivate = false;
	config_go.formation = formation;
	config_go.formationRotation = 0;

	// LOOOOL
	Kp_formation = 0.5;




	// Set up periodic
	periodic_info timerInfo;

	// Set up servo connection
	RobotClient robot;




	// Set up camera connection
	CameraClient camera;
	camera.open();

	// Issue a reset
	cam_msg_t dummy;
	// Resets obstacles and target...
	// Should not be needed when deactivating obstacles...
	//camera.request(&dummy, NONE, -1);

	// Set up connection to particle adam stuff
	ParticleClient laser;
	StateMessage laserMsg;


	// Loggger
	Logger logger;
	logger.init();



	cout << "Starting loop. " << endl;

	//Robot robot;
	//robot.init();



	// INIT matlab
	Controller_initialize();
	//Kp = 5;
	// Start periodic timer

	UARTChannel xbee;
	xbee.configure(UART_2);
	xbee.open(); // Keep open


	int counter = 0;
	make_periodic(1E5, &timerInfo);
	while(1) {
		/*
		 * INIT
		 */

		logger.timestamp();

		// Set state
		if(activateMission) {
			switch(state) {
				case INIT:
					runConfig = config_init;
					// check if last time was formation reached
					if(Controller_Y.formation_reached == 1) {
						state = GO;
						runConfig = config_go;
					}


					break;
				case GO:
					runConfig = config_go;
					break;
			}
			logger.add("state", static_cast<double>(state));
		}
		else {
			runConfig = config_default;
		}

		/*
		 * Request and set incomming sensor data
		 */



		robot_t me;
		camera.request(&me, robotId);






		cout << "Camera got: " << me.pos.x << " - " << me.pos.y << endl;
		cout << "Aref: " << me.aref.x << " - " << me.aref.y << endl;


		robot_t others[4];
		for(int i = 0; i < 4; i++) {
			camera.request(&others[i], OBSERVED_POS, i+1);
		}

		obstacle_t obstacles[3];
		for(int i = 0; i < 3; i++) {
			camera.request(&obstacles[i], OBSTACLE, i);
		}

		target_t target;
		camera.request(&target, TARGET);

		velocity_t mission_v;
		camera.request(&mission_v);	// This must be added to log later, as robot 1 generates its own mission v


		logger.add("X", me.pos.x);
		logger.add("Y", me.pos.y);
		logger.add("pos_valid", me.valid);
		logger.add("aref_X", me.aref.x);
		logger.add("aref_Y", me.aref.y);
		logger.add("angle_valid", me.valid);

		logger.add("target_X", target.pos.x);
		logger.add("target_Y", target.pos.y);

		logger.add("obstacle_X", 0.0);
		logger.add("obstacle_Y", 0.0);
		logger.add("obstacle_valid", false);


		/*
		 * Set input to matlab
		 */

		Controller_U.camera.pos.point[0] = me.pos.x;
		Controller_U.camera.pos.point[1] = me.pos.y;
		Controller_U.camera.pos.valid = me.valid;

		Controller_U.camera.aref.point[0] = me.aref.x;
		Controller_U.camera.aref.point[1] = me.aref.y;
		Controller_U.camera.aref.valid = me.valid;

		for(int i = 0; i < 4; i++) {
			Controller_U.all_robots[i].point[0] = others[i].pos.x;
			Controller_U.all_robots[i].point[1] = others[i].pos.y;
		}

		for(int i = 0; i < 3; i++) {
			Controller_U.obstacles[i].point[0] = obstacles[i].pos.x;
			Controller_U.obstacles[i].point[1] = obstacles[i].pos.y;
			Controller_U.obstacles[i].valid = obstacles[i].valid;
		}

		Controller_U.target.point[0] = target.pos.x;
		Controller_U.target.point[1] = target.pos.y;


		Controller_U.mission_v[0] = mission_v.vel.x;
		Controller_U.mission_v[1] = mission_v.vel.y;


		Controller_U.agent_index = robotId;

		/*
		 * Run controller step
		 */

		Controller_step();

		/*
		 * Logging and printing
		 */



		// Add matlablog to logfile.
		logger.addTable(logEntry, 10, "mlog");

		// Add estimated position to logfile
		logger.addTable(Controller_Y.current_pos, 2, "observed_pos");

		// Add observed heading to log
		logger.add("heading", static_cast<double>(Controller_Y.heading));

		// Add servos to log
		logger.addTable(Controller_Y.servo_out, 3, "servo");

		// Add formation reached to log
		logger.add("formation_reaced", static_cast<bool>(Controller_Y.formation_reached));

		// Add observer valid to log
		logger.add("observer_valid", static_cast<bool>(Controller_Y.observer_valid));
		// Send servodata to the servocontroller
		robot.send(Controller_Y.servo_out, 3);

		if(robotId == 1 || robotId == 2) {
			char outBuffer[10];
			outBuffer[0] = 'A';
			outBuffer[1] = 'B';
			outBuffer[2] = (int8_t) Controller_Y.servo_out[0];
			outBuffer[3] = static_cast<int8_t>(Controller_Y.servo_out[1]);
			outBuffer[4] = static_cast<int8_t>(Controller_Y.servo_out[2]);
			outBuffer[5] = 'C';
			outBuffer[6] = 'D';
			outBuffer[7] = '\0';
			// Send test stuff over xbee
			xbee.write(string(outBuffer));

		}


		/*
		 * Handle mission Velocity
		 */

		if(robotId == 1 && runConfig.activateLeaderV == true) {
			// Distribute and log
			// Broadcast position data to other systems
			Json::Value jsonMissionV;
			jsonMissionV["type"] = "common_v";
			jsonMissionV["velocity"]["x"] = Controller_Y.common_v[0];
			jsonMissionV["velocity"]["y"] = Controller_Y.common_v[1];
			jsonMissionV["robot_id"] = robotId;

			comm.send(jsonMissionV.toStyledString());

			// add this to log
			logger.add("mission_v_x", Controller_Y.common_v[0]);
			logger.add("mission_v_y", Controller_Y.common_v[1]);

		}
		else {
			// log received
			logger.add("mission_v_x", mission_v.vel.x);
			logger.add("mission_v_y", mission_v.vel.y);
		}


		// Add measured obstacle position to log
		char obstacleNameBuffer[20];
		for(int i = 0; i< 3; i++) {
			sprintf(obstacleNameBuffer, "obstacle_%d_X", i+1);
			logger.add(obstacleNameBuffer, obstacles[i].pos.x);

			sprintf(obstacleNameBuffer, "obstacle_%d_Y", i+1);
			logger.add(obstacleNameBuffer, obstacles[i].pos.y);

			sprintf(obstacleNameBuffer, "obstacle_%d_valid", i+1);
			logger.add(obstacleNameBuffer, obstacles[i].valid);
		}



		// Cout output
		cout << "Matlab got angle: " << Controller_Y.heading << endl;
		cout << "Current pos: " << Controller_Y.current_pos[0] << " - " <<  Controller_Y.current_pos[1]<< endl;
		cout << "Aref: " << me.aref.x << " - " << me.aref.y << endl;
		cout << "Servo: " << Controller_Y.servo_out[0] << " - " << Controller_Y.servo_out[1] << " - " << Controller_Y.servo_out[2] << endl;
		cout << "Blue: " << Controller_U.obstacles[0].point[0] << " - " << Controller_U.obstacles[0].point[1] << endl;

		// Send data to laser
		laserMsg.x = float(Controller_Y.current_pos[0]);
		laserMsg.y = float(Controller_Y.current_pos[1]);
		laserMsg.heading = float((M_PI/180)*logEntry[0]);

		laser.send(laserMsg);


		// Broadcast position data to other systems
		Json::Value myPositionMessage;
		myPositionMessage["type"] = "position_update";
		myPositionMessage["position"]["x"] = Controller_Y.current_pos[0];
		myPositionMessage["position"]["y"] = Controller_Y.current_pos[1];
		myPositionMessage["robot_id"] = robotId;

		comm.send(myPositionMessage.toStyledString());

		logger.execTimestamp();

		// Actually do the writing

		logger.write();

		// Check if done?
		bool done = false;
		if(robotId == 1) {
			if(Controller_Y.formation_reached == 1 && false) {
				// Send message to stop
				Value jsonMsg;
				jsonMsg["command"] = "stop";

				comm.send(jsonMsg.toStyledString());
				done = true;

				cout << "Robot 1: Formation reached" << endl;
			}

			// Check if anything on stdin
			fd_set descriptorSet;
			struct timeval tv;
			tv.tv_sec = 0; tv.tv_usec = 0;
			FD_ZERO(&descriptorSet);
			FD_SET(0, &descriptorSet);

			if(select(1, &descriptorSet, NULL, NULL, &tv)) {
				// fd is set
				// Easy solution, just send stop and break
				// Send message to stop
				cout << "Got something on cin, exiting" << endl;
				string input;
				//cin >> input;
				Value jsonMsg;
				jsonMsg["command"] = "kill";

				comm.send(jsonMsg.toStyledString());
				keepProcessRunning = false;
				done = true;
			}




		}

		// Check for stop message
		Value message = messageClient.requestJson();
		if(getCommand(message) == STOP) {
			done = true;
		}
		if(getCommand(message) == KILL) {
			done = true;
			keepProcessRunning = false;
			cout << "Got kill command." << endl;
		}

		if(getCommand(message) == UPDATE) {
			// Updating something...

			config_default.formationRotation = message.get("formationrotation", 0).asInt();
			cout << "Got update.." << runConfig.formationRotation << endl;
		}



		counter++;

		if(done) {
			// Stop robot, return
			cout << "Got done. " << endl;
			robot.send(0,0,0);

			if(robotId == 1 || robotId == 2) {
				char outBuffer[10];
				outBuffer[0] = 'A';
				outBuffer[1] = 'B';
				outBuffer[2] = 0;
				outBuffer[3] = 0;
				outBuffer[4] = 0;
				outBuffer[5] = 'C';
				outBuffer[6] = 'D';
				outBuffer[7] = '\0';
				// Send test stuff over xbee
				xbee.write(string(outBuffer, 8));

			}

			break;
		}

		// Wait untill next timer event.
		wait_period(&timerInfo);
	} // end loop

	}
	return 0;
}

// this little thread reads the comlink and is able to parse requests from
// the main thread
void* commReceiver(void* arg) {

	return NULL;
}



