/*
 * main.cpp
 *
 *  Created on: Mar 4, 2013
 *      Author: adamleon
 */
#include "timer.h"
#include <math.h>
#include "uart_channel.h"
#include "particle_controller.h"
#include "logger.h"
#include "analog_laser.h"
#include "particle.h"

#include <vector>
#include <sstream>
#include <iostream>
#include <errno.h>
#include <string>
#include <sys/fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#define PI 3.141529
#define MAP_RADIUS 2.0
#define PARTICLE_COUNT 10000
#define MEASUREMENT_LENGTH 0.025
#define UPDATE_UNSERTAINTY 0.999
using namespace std;

float prev_x, prev_y;

int main(int argc, char** argv) {
	AnalogLaser laser(AIN0);

	cout << "laser_read: begin" << endl;

	// Timer info
	periodic_info timerInfo;

	// ParticleClient
	ParticleHost host;

	// Logger
	Logger logger("laser_sim");
	logger.init();

	make_periodic(1E5, &timerInfo);

	while(1) {
		int array_size = 25;
		int mes[array_size];
		for(int i = 0; i < array_size; i++) {
			mes[i] = laser.measureDistance();
			usleep(1000);
		}


		int i, j, temp;

		for (i = (array_size - 1); i > 0; i--)
		{
		    for (j = 1; j <= i; j++)
		    {
		    	if (mes[j-1] > mes[j])
				{
		    		temp = mes[j-1];
				    mes[j-1] = mes[j];
				    mes[j] = temp;
				}
			}
		}

		int m = 0;
		for(i = 0; i < 5; i++) {
			m += mes[10 + i];
		}

		m /= 5;

		if(host.numberOfMessages()) {
			StateMessage state;
			state = host.receive();

			logger.timestamp();

			logger.add("X", state.x);
			logger.add("Y", state.y);

			logger.add("Heading", state.heading);
			logger.add("Laser", (double)m);
			logger.write();
		}

		// Wait until next timer event.
		wait_period(&timerInfo);
	}

	cout << "laser_read: end" << endl;
	return 0;
}
