#include "Robot.h"
#include "json/json.h"

#include <iostream>
#include <math.h>

using std::endl;
using std::cout;

using Json::Value;

/* This functions populates and updates the given robot */
void Robot::populate(map<COLOR_e, vector<Dot> > &all) {
	// Will be simple for now, but should later take into account
	// the possibility of other robots using same color for aref, 
	// and then take the closest aref for usage. 
	
	// First, do count


	if(all.count(this->cPosition) > 0 && all.at(this->cPosition).size() > 0) {
		// Get the first, will be the biggest as these are sorted by size
		this->dPosition = all.at(this->cPosition).at(0);
		
	}
	else {
		// Tried to pupulate, but could not find any dots. Setting current as invalid
		this->dPosition.valid = false;
	}

	if(all.count(this->cAref) > 0 && all.at(this->cAref).size() > 0) {
		if(this->bFindNearestAref) {

			// get distance to first element, for benchmark.
			float fNearest = distance(this->dPosition, all.at(this->cAref).at(0));
			int index = 0;

			// Loop all possibilities.			
			for(int i = 0; i < all.at(this->cAref).size(); i++) {
				float fDistance = distance(this->dPosition, all.at(this->cAref).at(i));

				if(fDistance < fNearest) {
					// this one is closer than current best
					index = i;
					fNearest = fDistance;
				}

			}

			// Now, index is the closest!
			this->dAref = all.at(this->cAref).at(index);

		}
		else {
			this->dAref = all.at(this->cAref).at(0);
		}
	}
	else {
		// Tried to populate, but no valid dots. Setting current to invalid
		this->dAref.valid = false;
	}






}

// this function generates 
void Robot::send(Comm& comm, bool fake) {

	// Only send if both valid!
	if(!(this->dAref.valid && this->dPosition.valid)) {
		return;
	}

	Value root;
	root["type"] = "camera";
	root["robot_id"] = this->iMyId;
	root["object_type"] = "robot";

	
	

	Value position;
	position["x"] = this->dPosition.pos.x;
	position["y"] = this->dPosition.pos.y;
	position["valid"] = this->dPosition.valid && !fake;

	root["position"] = position;

	Value aref;
	aref["x"] = this->dAref.pos.x;
	aref["y"] = this->dAref.pos.y;
	aref["valid"] = this->dAref.valid;
	root["aref"] = aref;
	
	comm.send(root.toStyledString());

}

float Robot::distance(Dot &dot1, Dot &dot2) {
	return sqrt( pow(dot1.pos.x - dot2.pos.x,2) + pow( dot1.pos.y - dot2.pos.y, 2));
}

int Robot::getCurrentAngle() {
	// Simply takes the atan2 argument!
	return static_cast<int>((180/3.14)*atan2(this->dAref.pos.y - this->dPosition.pos.y, this->dAref.pos.x - this->dPosition.pos.x));
}

void Robot::printAngle() {
	cout << "Robot " << this->iMyId << "s angle: " << this->getCurrentAngle() << endl;
}

// takes everything of a color, and sends it over udp
void Obstacle::send(Comm& comm, map<COLOR_e, vector<Dot> > &all) {


	if(all.count(this->cColor) > 0) {



		Value root;
		root["type"] = "camera";
		root["object_type"] = "obstacle";

		Value obstacles;

		for(int i = 0; i < all.at(this->cColor).size(); i++) {

			

			Value pos;
			pos["x"] = all.at(this->cColor).at(i).pos.x;
			pos["y"] = all.at(this->cColor).at(i).pos.y;
			pos["valid"] = all.at(this->cColor).at(i).valid;

			//root["position"] = pos;
			obstacles[i] = pos;

			
		}

		root["obstacles"] = obstacles;
		comm.send(root.toStyledString());
	}
}


// takes the biggest of the color
void Target::send(Comm& comm, map<COLOR_e, vector<Dot> > &all) {

	if(all.count(this->cColor) > 0) {
		


		if(all.at(this->cColor).size() > 0) {
			Value root;
			root["type"] = "camera";
			root["object_type"] = "target";

			Value pos;
			pos["x"] = all.at(this->cColor).at(0).pos.x;
			pos["y"] = all.at(this->cColor).at(0).pos.y;
			pos["valid"] = all.at(this->cColor).at(0).valid;

			root["position"] = pos;

			comm.send(root.toStyledString());
		}
	}
}
