#pragma once

/* Simple interface class to contain information about
	the position of a robot.
	Contains two dots, one for position and one for angle reference */


#include "types.h"
#include <map>
#include <vector>
#include "udp_interface.h"

using std::map;
using std::vector;

class Obstacle {
public:
	Obstacle(COLOR_e cColor):cColor(cColor) {};
	void send(Comm& comm, map<COLOR_e, vector<Dot> > &all);

private:
	COLOR_e cColor;
};

class Target {
public:
	Target(COLOR_e cColor):cColor(cColor) {};
	void send(Comm& comm, map<COLOR_e, vector<Dot> > &all);

private:
	COLOR_e cColor;

};




class Robot
{
public:
	Robot(int iMyId, COLOR_e cPosition, COLOR_e cAref, bool nearest = true):iMyId(iMyId), 
		cPosition(cPosition), cAref(cAref), bFindNearestAref(nearest) 
		{/* Intentionally empty */;};


	void populate(map<COLOR_e, vector<Dot> > &all);
	void send(Comm& comm, bool fake = false);
	int getCurrentAngle();
	void printAngle();
	

private:
	int iMyId;
	COLOR_e cPosition;
	COLOR_e cAref;
	bool bFindNearestAref;

	Dot dPosition;
	Dot dAref;

	float distance(Dot &dot1, Dot &dot2);
};

