#include "udp_interface.h"

#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"

#ifdef _WIN32
#include "libfreenect.hpp"
#endif

#include <opencv2/opencv.hpp>
#include <stdlib.h>
#include <stdio.h>


#include <string>

#include <iostream>
#include <sstream>

#include <vector>
#include <map>
#include <algorithm>    // std::sort

#include "types.h"
#include "ColorParameter.h"

#include <math.h>

#include "json/json.h"
#include "Robot.h"

#ifdef _WIN32
#define M_PI 3.14159265358979323846 // Becouse microsoft says fuck you.
#endif


/* SOME TEST CONFIG */

#define USE_KINECT

//#define DEBUG
#define SIMPLE


#if defined(USE_KINECT) && defined(_WIN32)
	#include "kinect.h"
#else
	#include "webCam.h"
#endif
#include "recorder.h"

using namespace cv;
using namespace std;
using namespace Json;

bool dotSort(Dot i, Dot j) {
	return (i.mass > j.mass); // Want to sort largest first.
}
float const SCALE_PX_M = 2/640.0;

/// Global variables
vector<Dot> reds, blues, greens, magentas, cyans, yellows;
		
map<COLOR_e, vector<Dot> > all;


ColorParameter blue(BLUE);
ColorParameter green(GREEN);
ColorParameter yellow(YELLOW);
ColorParameter magenta(MAGENTA);
ColorParameter cyan(CYAN);
ColorParameter red(RED);


Mat Hmatrix;


int countourColor = 0;


int thresh_hue_lower = 0;
int thresh_hue_upper = 255;

int thresh_sat_lower = 0;
int thresh_sat_upper = 255;

int thresh_val_lower = 0;
int thresh_val_upper = 255;

int canny_val_lower = 50;
int canny_val_upper = 200;

int const max_canny = 4000;

int const max_hue = 180;
int const max_sat = 255;
int const max_val = 255;




Scalar getLower(int lowerHue) {
	return Scalar(lowerHue, 90, 40);
}

Scalar getUpper(int upperHue) {
	return Scalar(upperHue, 255, 255);
}


int const max_value = 255;
int const max_type = 4;
int const max_BINARY_value = 255;

Mat src, src_hsv, dst, src_gray;
char* window_name = "Threshold Demo";

char* trackbar_type = "Type: \n 0: Binary \n 1: Binary Inverted \n 2: Truncate \n 3: To Zero \n 4: To Zero Inverted";
char* trackbar_value = "Value";

char* trackbar_hue_lower = "Hue: Lower";
char* trackbar_hue_upper = "Hue: Upper";



/// Function headers
void Threshold_Demo( int, void* );

void HomogenTransform(Mat& src);
void findTransformMatrix();

void sendColor(Comm& comm, COLOR_e color, Dot& dot);

#ifndef _WIN32

double timeElapsed(timespec startTime);
struct timespec _tsSubtract(struct timespec & time1, struct timespec & time2);

#endif

#ifdef _WIN32
Mat rgbMat(Size(640,480),CV_8UC3,Scalar(0));
#else

Mat rgbMat(Size(640,480),CV_8UC3,Scalar(0));

//Mat rgbMat(Size(320,240),CV_8UC3,Scalar(0));
#endif
/**
 * @function main
 */
int main( int argc, char** argv )
{
	// Kinect test
	

	/* ROBOTS */
	vector<Robot> robots;
	
	robots.push_back(Robot(1, GREEN, YELLOW));
	robots.push_back(Robot(2, MAGENTA, YELLOW));
	robots.push_back(Robot(3, RED, YELLOW));
	robots.push_back(Robot(4, BLUE, YELLOW));
	
	
	/* OBSTACLES */
	vector<Obstacle> obstacles;
	//obstacles.push_back(Obstacle(RED));
	

	/* TARGETS */
	vector<Target> targets;
	targets.push_back(Target(CYAN));

   Comm udp;
   udp.init(1337);


   // This window shows the original captured video
	namedWindow("rgb",CV_WINDOW_AUTOSIZE);
	device.startVideo();



 
	device.getVideo(rgbMat);


	rgbMat.copyTo(src);
	//cap >> src;
	cout << "Cols: " << src.cols << " - Rows: " << src.rows << endl;
  // COM STUFF
  //com_configure();

  // Set the global trransform matrix.
  findTransformMatrix();

  namedWindow("Circles", 1);
#ifdef DEBUG
  namedWindow("Video", 1);
  
  namedWindow( "Contours", CV_WINDOW_AUTOSIZE );
#endif

  /// Convert the image to HSV
  cvtColor( src, src_hsv, CV_BGR2HSV );


#ifdef DEBUG

  /// Create a window to display results
  namedWindow( window_name, CV_WINDOW_AUTOSIZE );
  
  /// Create Trackbar to choose type of Threshold
  createTrackbar( trackbar_hue_lower,
                  window_name, &thresh_hue_lower,
                  max_hue, Threshold_Demo );

  createTrackbar( trackbar_hue_upper,
                  window_name, &thresh_hue_upper,
                  max_hue, Threshold_Demo );

  createTrackbar( "Sat: Lower",
				  window_name, &thresh_sat_lower,
                  max_sat, Threshold_Demo );
  createTrackbar( "Sat: Upper",
				  window_name, &thresh_sat_upper,
                  max_sat, Threshold_Demo );

  createTrackbar( "Value: Lower",
				  window_name, &thresh_val_lower,
                  max_val, Threshold_Demo );
  createTrackbar( "Value: Upper",
				  window_name, &thresh_val_upper,
                  max_val, Threshold_Demo );

  createTrackbar( "Canny: Lower",
	  "Circles", &canny_val_lower,
	  max_canny, Threshold_Demo);
  createTrackbar( "Canny: Upper",
	  "Circles", &canny_val_upper,
	  max_canny, Threshold_Demo);

#endif

  /// Call the function to initialize
  Threshold_Demo( 0, 0 );

  /// Wait until user finishes program
  // with escape

  bool storing = false;
  bool recording = false;
  VideoWriter writer;
  while(true)
  {

	  
	
    int c;
    c = waitKey( 20 );
    if( (char)c == 27 )
      { break; }
	if( (char)c == 's' && storing == false) {
		storing = true;
		
		saveImage();

	}
	else {
		storing = false;
	}
	bool fake = false;
	if( (char)c == 'f') {
		// Fake the shit!
		fake = true;
		cout << "faking.." << endl;
	}
	if( (char)c == 'q') {
		cout << "Started recording.." << endl;
		// start to record
		startRecording();
	}
	if( (char)c == 'w') {
		// stop recording
		stopRecording();

	}




	// Actually do the processing
	Threshold_Demo(0,0);

	// Do some other silly stuff like sending 
	//cout << "Green: " << lastGreen.x << " - " << lastGreen.y << "Yellow: " << lastYellow.x << " - " << lastYellow.y;
	//cout << "Angle: " << atan2(lastYellow.y - lastGreen.y, lastYellow.x - lastGreen.x)*180/M_PI << endl;

	if(all[GREEN].size() == 0) {
		cout << "No green!" << endl;
	}
	if(all[GREEN].size() > 0 && fake) {
		all[GREEN].at(0).valid = false;
	}

	// Populate test
	for(unsigned int i = 0; i < robots.size() ; i++) {
		robots[i].populate(all);
		robots[i].printAngle();
		robots[i].send(udp); // add fake...?
	}

	if(obstacles.size()>0) {
		obstacles[0].send(udp, all);
	}
	if(targets.size() > 0) {
		targets[0].send(udp, all);
	}
	
	
   }
  
  device.stopVideo();
  return 0;

}



vector<Dot> findCircle(const Mat& src_hsv,ColorParameter const &color, Mat &contourImage) {
	// Start taking the time!
#ifndef _WIN32
	
        timespec time_start;
        clock_gettime(CLOCK_MONOTONIC, &time_start);
	
	cout << "Start: " << timeElapsed(time_start);
#endif


	// First, check if we should do special red inverse stuff
	Mat dst, canny_output;

	Scalar lower = color.getLower();
	Scalar upper = color.getUpper();

	if(lower[0] > upper[0]) {
		// Special, red "Wraps around" hue. 
		// Solution: Shift everything hue by the somewhat random number 60
		const int shifter = 60;

		// What we do instead, (becouse of shit with wrap around, 
		// perform two operations and or them. 
		Scalar newLower = lower;
		
		Mat tmp;
		inRange(src_hsv, Scalar(0, lower[1], lower[2]), upper, tmp);
		inRange(src_hsv, lower, Scalar(180, upper[1], upper[2]), dst);

		bitwise_or(tmp, dst, dst);
		

	}
	else{
		inRange(src_hsv, lower, upper,	dst);
	}

	// Erode! (shrink)

	int erosion_size = color.getErosionSize();;
  Mat element_erosion = getStructuringElement( MORPH_RECT,
                                       Size( 2*erosion_size + 1, 2*erosion_size+1 ),
                                       Point( erosion_size, erosion_size ) );

	

	int dilation_size = color.getDilationSize();
   Mat element_dilation = getStructuringElement( MORPH_RECT,
                                       Size( 2*dilation_size + 1, 2*dilation_size+1 ),
                                       Point( dilation_size, dilation_size ) );


   if(color.doDilationFirst()) {
		dilate(dst, dst, element_dilation); 
		erode(dst, dst, element_erosion);

		if(color.getSecondDilationSize() > 0) {
			dilation_size = color.getSecondDilationSize();
			element_dilation = getStructuringElement( MORPH_RECT,
                                       Size( 2*dilation_size + 1, 2*dilation_size+1 ),
                                       Point( dilation_size, dilation_size ) );

			dilate(dst, dst, element_dilation); 
		}

   }
   else {
		erode(dst, dst, element_erosion);
		dilate(dst, dst, element_dilation); 
   }
   
   	// Done eroding shit
#ifndef _WIN32
	cout << "Erode: " << timeElapsed(time_start);
#endif

	// Canny to find each circle
	// Then, moments to find masses.

	// Canny to find edges. 
	//Canny(dst, canny_output, canny_val_lower,canny_val_upper);
	Canny(dst, canny_output, 1,2);

#ifndef _WIN32
	cout << "Canny: " << timeElapsed(time_start);
#endif
	// Try to add
	add(contourImage, color.getDrawColor(), contourImage, canny_output);


	// List of countours
	vector<vector<Point> > contours;

	/// Find contours
	findContours( canny_output, contours, CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE, Point(0, 0) );

#ifndef _WIN32
	cout << "Add and countours " << timeElapsed(time_start);
#endif
	/// Get the moments
	vector<Moments> mu(contours.size() );
	for( int i = 0; i < contours.size(); i++ )
		{ mu[i] = moments( contours[i], false ); }

	///  Get the mass centers:
	vector<Dot> mc;
	for( int i = 0; i < contours.size(); i++ ){ 
		// Check if area is greater than a given threshold
		//cout << "Area: " << mu[i].m00 << endl;

		// Push back!
		Dot dot;
		dot.mass = mu[i].m00;
		dot.pos = Point2f( mu[i].m10/mu[i].m00 , mu[i].m01/mu[i].m00);
		dot.valid = true;
		dot.found = true;

		mc.push_back(dot);
		/*
		if(mu[i].m00 > 220) {
			mc.push_back( Point2f( mu[i].m10/mu[i].m00 , mu[i].m01/mu[i].m00 ));
		}
		*/
	}

	// Sort
	sort(mc.begin(), mc.end(), dotSort);
#ifndef _WIN32
	cout << "Done: " << timeElapsed(time_start);
#endif
	//cout << "Mass centers: " << mc << endl;
	return mc;

}




/**
 * @function Threshold_Demo
 */
void Threshold_Demo( int, void* )
{


	device.getVideo(rgbMat);
	cv::imshow("rgb", rgbMat);

	rgbMat.copyTo(src);
	cout << "Captured, starting to process..." << endl;

	warpPerspective(src, src, Hmatrix, src.size());

	cvtColor( src, src_hsv, CV_BGR2HSV );


	Mat drawing = Mat::zeros(src.size(), CV_8UC3);
	Mat contours = Mat::zeros(src.size(), CV_8UC3);


	// do stuff
	
	reds		= findCircle(src_hsv, red,		contours);
	cout << "1 "<<endl;
	magentas	= findCircle(src_hsv, magenta,	contours);	
	cout << "2 "<<endl;
	cyans		= findCircle(src_hsv, cyan,		contours);	
	cout << "3 "<<endl;
	blues		= findCircle(src_hsv, blue,		contours);	
	cout << "4 "<<endl;
	greens		= findCircle(src_hsv, green,	contours);
	cout << "5 "<<endl;
	yellows		= findCircle(src_hsv, yellow,	contours);
	cout << "6 "<<endl;

	cout << "Got all colours" << endl;
#ifdef DEBUG
	inRange(src_hsv, Scalar(thresh_hue_lower, thresh_sat_lower, thresh_val_lower),
					Scalar(thresh_hue_upper, thresh_sat_upper, thresh_val_upper),
					dst);
	imshow("Contours", contours);
	add(contours, src, src);

#endif
	// append to single vector (for drawing)

	all[BLUE] = blues;
	all[RED] = reds;
	all[GREEN] = greens;
	all[MAGENTA] = magentas;
	all[CYAN] = cyans;
	all[YELLOW] = yellows;


	/// Show in a window
	// Draw it

	for(int color = 0; color < 6 ; color++) {
		for(int i = 0; i < all[static_cast<COLOR_e>(color)].size(); i++) {
			// all but blue draw more than one
			if(i < 1 || color != (int)BLUE) {
				circle( drawing, all[static_cast<COLOR_e>(color)].at(i).pos, 8, COLORS[color], 2);
			}
		}
	}

	// Transform to meters
	for(int color = 0; color < 6 ; color++) {
		for(int i = 0; i < all[static_cast<COLOR_e>(color)].size(); i++) {
			all[static_cast<COLOR_e>(color)].at(i).pos*=SCALE_PX_M;
		}
	}


	imshow("Circles", drawing);

#ifdef DEBUG

	imshow("Video",  src);
	imshow( window_name, dst );
#endif
	cout << "Processing complete. " << endl;
}



void findTransformMatrix() {
	/* Do a linear transformation to get position in meters */

	Point2f srcTri[4];
	Point2f dstTri[4];

	
	/// Set the dst image the same type and size as src
   Mat warp_dst = Mat::zeros( src.rows, src.cols, src.type() );
   Mat warp_mat( 4,4, CV_32FC1 );

   /// Set your 4 points to calculate the  Affine Transform
   srcTri[0] = Point2f( 0,0 );
   srcTri[1] = Point2f( 0, src.rows );
   srcTri[2] = Point2f( src.cols - 1, src.rows );
   srcTri[3] = Point2f( src.cols - 1, 0 );

  

   dstTri[0] = Point2f( src.cols*0.0, src.rows*0);
   dstTri[1] = Point2f( src.cols*0.125, src.rows*1);
   dstTri[2] = Point2f( src.cols*0.875, src.rows*1 );
   dstTri[3] = Point2f( src.cols*1, src.rows*0 );



   vector<Point2f> srcPoints;
   vector<Point2f> dstPoints;
   srcPoints.assign(srcTri, srcTri+4);
   dstPoints.assign(dstTri, dstTri+4);

   Hmatrix = findHomography(srcPoints, dstPoints, CV_RANSAC);
}

#ifndef _WIN32



struct  timespec  _tsSubtract (
        struct  timespec & time1,
        struct  timespec & time2)
{    /* Local variables. */
    struct  timespec  result ;

/* Subtract the second time from the first. */

    if ((time1.tv_sec < time2.tv_sec) ||
        ((time1.tv_sec == time2.tv_sec) &&
         (time1.tv_nsec <= time2.tv_nsec))) {           /* TIME1 <= TIME2? */
        result.tv_sec = result.tv_nsec = 0 ;
    } else {                                            /* TIME1 > TIME2 */
        result.tv_sec = time1.tv_sec - time2.tv_sec ;
        if (time1.tv_nsec < time2.tv_nsec) {
            result.tv_nsec = time1.tv_nsec + 1000000000L - time2.tv_nsec ;
            result.tv_sec-- ;                           /* Borrow a second. */
        } else {
            result.tv_nsec = time1.tv_nsec - time2.tv_nsec ;
        }
    }

    return (result) ;

}

double timeElapsed(timespec startTime) {
        timespec time_now;
        clock_gettime(CLOCK_MONOTONIC, &time_now);
        timespec time_diff =  _tsSubtract(time_now, startTime);


        double diff = time_diff.tv_sec + time_diff.tv_nsec / double(1.0E9);

        return diff;

}
#endif

